'''
custom_print module can handle any type of variable.
'''

#pylint: disable=bare-except
#pylint: disable=invalid-name
#pylint: disable=unused-import
#pylint: disable=line-too-long
#pylint: disable=too-many-lines
#pylint: disable=no-else-return
#pylint: disable=unused-variable
#pylint: disable=too-many-locals
#pylint: disable=protected-access
#pylint: disable=too-many-branches
#pylint: disable=consider-using-in
#pylint: disable=chained-comparison
#pylint: disable=too-many-arguments
#pylint: disable=too-many-statements
#pylint: disable=multiple-statements
#pylint: disable=consider-using-join
#pylint: disable=unspecified-encoding
#pylint: disable=unnecessary-negation
#pylint: disable=singleton-comparison
#pylint: disable=too-few-public-methods
#pylint: disable=too-many-nested-blocks
#pylint: disable=too-many-public-methods
#pylint: disable=expression-not-assigned
#pylint: disable=consider-using-enumerate
#pylint: disable=unnecessary-comprehension
#pylint: disable=too-many-return-statements
#pylint: disable=unbalanced-tuple-unpacking
#pylint: disable=consider-using-max-builtin
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-positional-arguments
#pylint: disable=inconsistent-return-statements
#pylint: disable=possibly-used-before-assignment


#-----------------------------------------------------------------------------------------------------------------------------------------------------
#12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
#        1         2         3         4         5         6         7         8         9         A         B         C         D         E         F
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Required Modules                                                                                                                                   -
#-----------------------------------------------------------------------------------------------------------------------------------------------------
from custom_print.ref_names import Align
from custom_print.fancy_message import FancyMessage
from custom_print.fancy_functions import move_cursor_right

#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Font Style Class. Manipulate Font In The Terminal                                                                                                 --
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# ansi codes
# reset_all : "\033[0m"         Terminal_Bell : "\a"
# bold_on   : "\033[1m"         underline_on  : "\033[4m"         hidden_on    : "\033[8m"
# bold_off  : "\033[22m"        underline_off : "\033[24m"        hidden_off   : "\033[28m"
# dim_on    : "\033[2m"         blinking_on   : "\033[5m"         strike_on    : "\033[9m"
# dim_off   : "\033[22m"        blinking_off  : "\033[25m"        strike_off   : "\033[29m"
# italic_on : "\033[3m"         reverse_on    : "\033[7m"         background   : "\033[48;5;{str(bg)}m"
# italic_off: "\033[23m"        reverse_off   : "\033[27m"        foreground   : "\033[38;5;{str(fg)}m"
# backspace : "\b"              horizontal tab: "\t"              vertical tab : "\v"
class FontStyle:
    '''
    FontStyle class print fancy text
    '''
    def __init__(self):
        # General Use
        self.bg  = -1;       self.bold      = False;    self.hidden    = False;    self.italic    = False
        self.fg  = -1;       self.underline = False;    self.strike    = False;    self.inverse   = False
        self.dim = False;    self.blinking  = False

        # Print_Style
        self.align = Align.JUSTIFY;    self.bg_top_lines    = 0
        self.forced_align = False;      self.bg_bottom_lines = 0

        # self.indent is used for style_on and for print_style when using justify
        self.indent    = 0

    def set_font_style(self)->str:

        '''  This function changes the attributes of the font (bold=bool, bg=int, fg=int).

        Colors range from -1 to 256, where -1 or 256 is the defualt one.  '''


        # bg_color and fg_color, are int values but we convert then to str values
        reset = "\033[0m"
        if self.bg < 0 or self.bg > 255:  bgc = "reset"
        else:                             bgc = str(self.bg)

        if self.fg < 0 or self.fg > 255:  fgc = "reset"
        else:                             fgc = str(self.fg)


        if (bgc == "reset" and fgc == "reset"):  settings = reset
        elif bgc == "reset" and fgc != "reset":  settings = reset+"\033[38;5;"+fgc+"m"
        elif bgc != "reset" and fgc == "reset":  settings = reset+"\033[48;5;"+bgc+"m"
        elif bgc != "reset" and fgc != "reset":  settings = reset+"\033[48;5;"+bgc+";38;5;"+fgc+"m"
        else:                                    settings = reset


        if   (self.bold == True and self.dim  == False): settings = settings + "\033[1m"
        elif (self.bold == True and self.dim  == True):  settings = settings + "\033[1m"
        elif (self.bold == False and self.dim == True):  settings = settings + "\033[2m"
        else:                                            pass   # (bold == False and dim == False):


        if self.italic == True:      settings = settings + "\033[3m"
        else:                        settings =  settings + "\033[23m"

        if self.underline == True:   settings = settings + "\033[4m"
        else:                        settings = settings + "\033[24m"

        if self.blinking == True:    settings = settings + "\033[5m"
        else:                        settings = settings + "\033[25m"

        if self.inverse == True:     settings = settings + "\033[7m"
        else:                        settings = settings + "\033[27m"

        if self.hidden == True:      settings = settings + "\033[8m"
        else:                        settings = settings + "\033[28m"

        if self.strike == True:      settings = settings + "\033[9m"
        else:                        settings = settings + "\033[29m"

        return settings



    def reset_style(self):
        '''
        Reset the FontStyle class
        '''
        # General Use
        self.bg  = -1;       self.bold      = False;    self.hidden    = False;    self.italic    = False
        self.fg  = -1;       self.underline = False;    self.strike    = False;    self.inverse   = False
        self.dim = False;    self.indent    = 0;        self.blinking  = False

        # print_style
        self.align = Align.JUSTIFY;    self.bg_top_lines     = 0
        self.forced_align = False;      self.bg_bottom_lines = 0



    def style_on(self)->str:
        '''
        Activate the style
        '''
        if self.indent <= 0:
            settings = self.set_font_style()# + f"\033[0C"
        else:                 settings = self.set_font_style() + f"\033[{str(self.indent)}C"
        return settings



    def style_off(self)->str:
        '''
        Deactivate the style
        '''
        return "\033[0m"



    def print_style(self, msg)->None:
        '''
        print_style will help to print a fancy statement on the terminal
        '''
        #---------------------------------------------------------------------------------------------------------------------------------
        def print_bg_lines(move_crs, insert_sp, settings, lines):
            if lines == 0:
                pass

            else:
                n = lines
                while n>0:
                    print(f"{move_crs}{settings}{insert_sp}\033[0m")
                    n -= 1

        #---------------------------------------------------------------------------------------------------------------------------------
        def terminal_cols_smaller_than_biggest_line():
            message_lst = msg.split()
            print(f"{settings}",end="")
            for l in range(len(message_lst)):
                for n in message_lst[l]:
                    print(f"{n}",end="")
                print(" ", end="")

            suma = 0
            for w in message_lst:
                suma += len(w) + 1

            if tncols > suma:
                diff = tncols - suma
                print(move_cursor_right(diff, True),end="")

            else:
                done = True
                while done:
                    suma = suma - tncols
                    if suma < 0:
                        done = False

                diff = suma * (-1)
                print(move_cursor_right(diff, True),end="")
            print("\033[0m",end="") # reset font


        #---------------------------------------------------------------------------------------------------------------------------------
        reset = "\033[0m"
        fm = FancyMessage()
        fm.left_indent = 0; fm.right_indent = 0
        tnrows, tncols, space_available, number_letter_line_list, adj_diff_space, new_msg, n_lines = fm.get_msg_attributes(msg,True)
        settings = self.set_font_style()

        cnt_l = 0     # counting the number of letter in the new message
        cnt_p = 0     # counting the position of the list containing the letters
        wd_line = ""  # keeps the line info
        wd_list = []  # keep the text of the lines as list

        for l in range(len(new_msg)):
            wd_line += new_msg[l]
            cnt_l += 1
            if cnt_l == number_letter_line_list[cnt_p]:
                cnt_l  = 0
                cnt_p += 1
                wd_list.append(wd_line)
                wd_line = ""

        biggets_line  = max(number_letter_line_list)
        bg_space_line = move_cursor_right(biggets_line,option_space=True)

        if biggets_line < tncols:
            #-----------------------------------------------------------------------------------------------------------------------------------------
            if self.align.lower() == Align.CENTER or self.align.lower() == "c":
            #-----------------------------------------------------------------------------------------------------------------------------------------
                move_cursor  = move_cursor_right(n=(int((tncols - biggets_line)/2)),option_space=False)
                print_bg_lines(move_cursor, bg_space_line , settings,self.bg_top_lines)
                if self.forced_align == True:
                    #---------------------------------------------------------------------------------------------------------------------------------
                    for l in wd_list:
                        l2 = int((biggets_line - len(l))/2)
                        r = int((biggets_line - len(l))%2)
                        print(f"{move_cursor}{settings}{move_cursor_right(n=l2,option_space=True)}{l}{move_cursor_right(n=l2+r,option_space=True)}{reset}")

                else:   # Center (force = False)
                    #---------------------------------------------------------------------------------------------------------------------------------
                    for l in wd_list:
                        adj = biggets_line - len(l)
                        print(f"{move_cursor}{settings}{l}{move_cursor_right(n=adj,option_space=True)}{reset}")
                print_bg_lines(move_cursor, bg_space_line , settings,self.bg_bottom_lines)


            #-----------------------------------------------------------------------------------------------------------------------------------------
            elif self.align.lower() == Align.RIGHT or self.align.lower() == "r":
            #-----------------------------------------------------------------------------------------------------------------------------------------
                move_cursor = move_cursor_right(n=(int(tncols - biggets_line)), option_space=False)
                print_bg_lines(move_cursor, bg_space_line , settings, self.bg_top_lines)
                if self.forced_align == True:
                    #---------------------------------------------------------------------------------------------------------------------------------
                    for l in wd_list:
                        l2 = int(tncols - biggets_line)
                        ll = biggets_line - len(l)
                        print(f"{move_cursor_right(n=l2,option_space=False)}{settings}{move_cursor_right(n=ll,option_space=True)}{l}{reset}")

                else:   # Right (forced = False)
                    #---------------------------------------------------------------------------------------------------------------------------------
                    for l in wd_list:
                        l2 = int(tncols - biggets_line)
                        ll = biggets_line - len(l)
                        print(f"{move_cursor_right(n=l2,option_space=False)}{settings}{l}{move_cursor_right(n=ll,option_space=True)}{reset}")

                print_bg_lines(move_cursor, bg_space_line , settings,self.bg_bottom_lines)


            #-----------------------------------------------------------------------------------------------------------------------------------------
            elif self.align.lower() == Align.LEFT or self.align.lower() == "l":
            #-----------------------------------------------------------------------------------------------------------------------------------------
                move_cursor = move_cursor_right(n=0, option_space=False)
                print_bg_lines(move_cursor, bg_space_line , settings, self.bg_top_lines)
                if self.forced_align == True:
                    for l in wd_list:
                        ll = biggets_line - len(l)
                        print(f"{settings}{l}{move_cursor_right(n=ll,option_space=True)}{reset}")

                else:   # Left (forced = False)
                    for l in wd_list:
                        ll = biggets_line - len(l)
                        print(f"{settings}{move_cursor_right(n=ll,option_space=True)}{l}{reset}")

                print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)


            #-----------------------------------------------------------------------------------------------------------------------------------------
            elif self.align.lower() == Align.JUSTIFY or self.align.lower() == "j":
            #-----------------------------------------------------------------------------------------------------------------------------------------
                move_cursor = move_cursor_right(n=self.indent, option_space=False)
                print_bg_lines(move_cursor, bg_space_line , settings, self.bg_top_lines)

                if self.forced_align == True:
                    for l in wd_list:
                        ll = biggets_line - len(l)
                        print(f"{move_cursor_right(n=self.indent,option_space=False)}{settings}{l}{move_cursor_right(n=ll,option_space=True)}{reset}")

                else:   # Justify (forced = False)
                    for l in wd_list:
                        ll = biggets_line - len(l)
                        print(f"{move_cursor_right(n=self.indent,option_space=False)}{settings}{move_cursor_right(n=ll,option_space=True)}{l}{reset}")

                print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)

            #-----------------------------------------------------------------------------------------------------------------------------------------
            else:
            #-----------------------------------------------------------------------------------------------------------------------------------------
                carry = 0
                if self.forced_align == True:
                    for l in range(len(number_letter_line_list)):
                        print(f"{move_cursor_right(self.indent,False)}{settings}",end="")
                        for n in range(number_letter_line_list[l]):
                            print(f"{new_msg[n+carry]}",end="")
                        carry += number_letter_line_list[l]
                        print(reset)
                else:
                    bg_space_line = move_cursor_right(tncols,option_space=True)
                    move_cursor = ""
                    print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)
                    terminal_cols_smaller_than_biggest_line()
                    print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)

        else:
            # It will come in only if the condition is = if biggets_line < tncols:
            bg_space_line = move_cursor_right(tncols,option_space=True)
            move_cursor = ""
            print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)
            terminal_cols_smaller_than_biggest_line()
            print_bg_lines(move_cursor, bg_space_line , settings, self.bg_bottom_lines)



'''
custom_print module can handle any type of variable.
'''

#pylint: disable=bare-except
#pylint: disable=invalid-name
#pylint: disable=unused-import
#pylint: disable=line-too-long
#pylint: disable=too-many-lines
#pylint: disable=no-else-return
#pylint: disable=wildcard-import
#pylint: disable=unused-variable
#pylint: disable=too-many-locals
#pylint: disable=protected-access
#pylint: disable=too-many-branches
#pylint: disable=consider-using-in
#pylint: disable=chained-comparison
#pylint: disable=too-many-arguments
#pylint: disable=too-many-statements
#pylint: disable=multiple-statements
#pylint: disable=consider-using-join
#pylint: disable=unspecified-encoding
#pylint: disable=unnecessary-negation
#pylint: disable=singleton-comparison
#pylint: disable=too-few-public-methods
#pylint: disable=too-many-nested-blocks
#pylint: disable=too-many-public-methods
#pylint: disable=expression-not-assigned
#pylint: disable=consider-using-enumerate
#pylint: disable=unnecessary-comprehension
#pylint: disable=too-many-return-statements
#pylint: disable=unbalanced-tuple-unpacking
#pylint: disable=consider-using-max-builtin
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-positional-arguments
#pylint: disable=inconsistent-return-statements
#pylint: disable=possibly-used-before-assignment


#-----------------------------------------------------------------------------------------------------------------------------------------------------
#12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
#        1         2         3         4         5         6         7         8         9         A         B         C         D         E         F
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Class Draw Pictures Around The Terminal                                                                                                           --
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
from custom_print.fancy_cursor import*
#from custom_print.ref_names import Move  # cursor already has Move
from custom_print.ref_names import Layout
from custom_print.ref_names import Line_Style
from custom_print.fancy_functions import ins_chr
from custom_print.fancy_functions import set_font
from custom_print.fancy_functions import reset_font
from custom_print.fancy_format import FancyFormat


class Pen(Cursor):                         # Inheritance the Cursor Class here.
    '''
    Pen class will draw lines nad squares
    '''
    def __init__(self):                    # Initializing Draw Class as self
        super().__init__()                 # Super Class to use all (vars and funs) from Cursor Class
                                           # with the Initialization Draw Class(self), ex. self.gotoxy(x,y)
        # General Section
        self.adj_indent = 0                # space from the terminal to the box
        self.draw_line_bold = False
        self.draw_line_bg = -1
        self.draw_line_fg = -1
        self.fill_color = False

        # Rectangle Section
        # Horizontal Line Section
        self.top_horizontal_line_chr = "-";      self.bottom_horizontal_line_chr = "-"
        # Vertical Line Section
        self.left_vertical_line_chr  = "|";      self.right_vertical_line_chr = "|"
        # Corner Section
        self.top_left_corner_chr     = "+";      self.top_right_corner_chr   = "+"
        self.bottom_right_corner_chr = "+";      self.bottom_left_corner_chr = "+"


    def draw_line(self, size=0, layout=Layout.HORIZONTAL, tail="\N{BLACK DIAMOND}", body="-", head="\N{BLACK DIAMOND}"):

        '''  It draws a line with the parameters specified

             draw_line(size=0, layout=Layout.HORIZONTAL,
             tail="\N{BLACK DIAMOND}", body="-", head="\N{BLACK DIAMOND}"  '''

        settings = set_font(self.draw_line_bold, self.draw_line_bg, self.draw_line_fg)

        if layout.lower() == "h" or layout.lower() == Layout.HORIZONTAL:
            self.jumpTo(qty = self.adj_indent, direction = Move.RIGHT)
            print(f"{settings}{tail}",end="")
            for n in range(size-2): print(body,end="")
            print(head,end="")
            reset_font()


        elif layout.lower() == "v" or layout.lower() == Layout.VERTICAL:
            self.jumpTo(qty = self.adj_indent, direction = Move.RIGHT)
            print(f"{settings}{tail}")
            for n in range(size-2): print(f"{self.moveTo(qty = self.adj_indent, direction = Move.RIGHT)}{body}")
            print(f"{self.moveTo(qty=self.adj_indent, direction=Move.RIGHT)}{head}")
            reset_font()

        else:  pass


    def draw_rectangle(self,length=3, width=3, style=Line_Style.DASH):

        '''  It draws a rectangle with the parameters specified
             draw_rectangle(self,length=3, width=3, style=Line_Style.DASH)  '''

        if length <= 2: length = 3   # length = largo, width = alto
        if width  <= 2: width  = 3

        #---------------------------------------------------------------------------------------------------------------
        # Refill bg Option For The Rectangle                                                                           -
        #---------------------------------------------------------------------------------------------------------------
        if self.fill_color == True:
            square = []

            sq_in = FancyFormat()
            # General
            sq_in.adj_indent = self.adj_indent
            sq_in.adj_space  = 0

            # Data section
            sq_in.data_bg = self.draw_line_bg
            sq_in.data_fg = self.draw_line_fg
            sq_in.data_all_cell_bg   = True

            # Horizontal Line Section
            sq_in.top_horizontal_line_chr    = self.top_horizontal_line_chr
            sq_in.bottom_horizontal_line_chr = self.bottom_horizontal_line_chr
            sq_in.top_horizontal_line_on     = True
            sq_in.bottom_horizontal_line_on  = True


            sq_in.horizontal_line_bold = self.draw_line_bold           # two values False and True (0 and 1)
            sq_in.horizontal_line_bg   = self.draw_line_bg             # values -1 to 255
            sq_in.horizontal_line_fg   = self.draw_line_fg             # values -1 to 255

            # Vertical Line Section
            sq_in.vertical_line_bold = self.draw_line_bold             # two values False and True (0 and 1)
            sq_in.vertical_line_bg   = self.draw_line_bg               # values -1 to 255
            sq_in.vertical_line_fg   = self.draw_line_fg               # values -1 to 255

            sq_in.left_vertical_line_chr  = self.left_vertical_line_chr
            sq_in.right_vertical_line_chr = self.right_vertical_line_chr

            # Corner Section
            sq_in.top_left_corner_chr     = self.top_left_corner_chr
            sq_in.top_right_corner_chr    = self.top_right_corner_chr
            sq_in.bottom_right_corner_chr = self.bottom_right_corner_chr
            sq_in.bottom_left_corner_chr  = self.bottom_left_corner_chr
            sq_in.all_corner_bold_chr = self.draw_line_bold       # two values False and True (0 and 1)
            sq_in.bg_corner_chr   = self.draw_line_bg         # values -1 to 255
            sq_in.fg_corner_chr   = self.draw_line_fg         # values -1 to 255

            # Line Under Header and Header Section
            sq_in.header_bg = self.draw_line_bg
            sq_in.header_fg = self.draw_line_fg

            sq_in.header_horizontal_line_on = False

            sq_in.header_all_cell_bg = True

            sq_in.header_vertical_line_bg_chr = self.draw_line_bg
            sq_in.header_vertical_line_fg_chr = self.draw_line_fg

            for n in range(width-2):
                square.append([ins_chr(length-2)])

            sq_in.print_fancy_format(square, style)

        #---------------------------------------------------------------------------------------------------------------
        # NO Refill bg Option For The Rectangle                                                                        -
        #---------------------------------------------------------------------------------------------------------------
        else:
            def print_horiz_sq_line(settings, indent, size, tail, body, head):
                self.jumpTo(qty = indent, direction = Move.RIGHT)
                print(f"{settings}{tail}",end="")
                for n in range(size-2): print(body,end="")
                print(head)
                reset_font()

            def print_vert_sq_line(settings, indent, size, tail, body, head):
                self.jumpTo(qty = indent, direction = Move.RIGHT)
                print(f"{settings}{tail}")
                for n in range(size-2): print(f"{self.moveTo(qty = indent, direction = Move.RIGHT)}{body}")
                print(f"{self.moveTo(qty=indent, direction=Move.RIGHT)}{head}")
                reset_font()


            if style.lower() == Line_Style.CUSTOMIZED: pass
            else:                                      # Backup all the default values
                # Horizontal Line Section
                thlc = self.top_horizontal_line_chr;    bhlc = self.bottom_horizontal_line_chr

                # Vertical Line Section
                lvlc = self.left_vertical_line_chr;     rvlc = self.right_vertical_line_chr

                # Corner Section
                tlcc = self.top_left_corner_chr;        trcc = self.top_right_corner_chr
                brcc = self.bottom_right_corner_chr;    blcc = self.bottom_left_corner_chr

                #---------------------------------------------------------------------------------------------------------------
                # start drwaing the rectangle                                                                                  -
                #---------------------------------------------------------------------------------------------------------------
                if style.lower() == Line_Style.SINGLE_LINE:

                    # Horizontal Line Section
                    self.top_horizontal_line_chr = "\u2500";      self.bottom_horizontal_line_chr="\u2500"

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u2502";      self.right_vertical_line_chr = "\u2502"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u250C";          self.top_right_corner_chr = "\u2510"
                    self.bottom_right_corner_chr="\u2518";        self.bottom_left_corner_chr="\u2514"


                elif style.lower() == Line_Style.SINGLE_BOLD:

                    # Horizontal Line Section
                    self.top_horizontal_line_chr = "\u2501";      self.bottom_horizontal_line_chr="\u2501"

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u2503";      self.right_vertical_line_chr = "\u2503"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u250F";          self.top_right_corner_chr = "\u2513"
                    self.bottom_right_corner_chr="\u251B";        self.bottom_left_corner_chr="\u2517"


                elif style.lower() == Line_Style.SINGLE_HEAVY:
                    # Horizontal Line Section
                    self.top_horizontal_line_chr = "\u2586";      self.bottom_horizontal_line_chr="\u2586"

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u2588";      self.right_vertical_line_chr = "\u2588"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u2586";          self.top_right_corner_chr = "\u2586"
                    self.bottom_right_corner_chr="\u2588";        self.bottom_left_corner_chr="\u2588"


                elif style.lower() == Line_Style.DOUBLE_LINE:
                    # Horizontal Line Section
                    self.top_horizontal_line_chr = "\u2550";      self.bottom_horizontal_line_chr="\u2550"

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u2551";      self.right_vertical_line_chr = "\u2551"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u2554";          self.top_right_corner_chr = "\u2557"
                    self.bottom_right_corner_chr="\u255D";        self.bottom_left_corner_chr="\u255A"


                elif style.lower() == Line_Style.SQ_BRACKETS:
                    # Horizontal Line Section
                    self.top_horizontal_line_chr = " ";           self.bottom_horizontal_line_chr=" "

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u2502";      self.right_vertical_line_chr = "\u2502"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u250C";          self.top_right_corner_chr = "\u2510"
                    self.bottom_right_corner_chr="\u2518";        self.bottom_left_corner_chr="\u2514"


                elif style.lower() == Line_Style.DASH:
                    # Horizontal Line Section
                    self.top_horizontal_line_chr = "\u002D";      self.bottom_horizontal_line_chr="\u002D"

                    # Vertical Line Section
                    self.left_vertical_line_chr  = "\u254E";      self.right_vertical_line_chr = "\u254E"

                    # Outside Corner Section
                    self.top_left_corner_chr = "\u002B";          self.top_right_corner_chr = "\u002B"
                    self.bottom_right_corner_chr="\u002B";        self.bottom_left_corner_chr="\u002B"


                elif style.lower() == Line_Style.NONE:
                    # Horizontal Line Section
                    self.top_horizontal_line_chr = " ";           self.bottom_horizontal_line_chr=" "

                    # Vertical Line Section
                    self.left_vertical_line_chr  = " ";           self.right_vertical_line_chr = " "

                    # Outside Corner Section
                    self.top_left_corner_chr = " ";               self.top_right_corner_chr = " "
                    self.bottom_right_corner_chr=" ";             self.bottom_left_corner_chr=" "

                else: pass
            #-------------------------------------------------------------------------------------------------------------------
            # def draw_rectangle(self,length=3, width=3, style=Line_Style.DASH):
            # def set_font(bold=False,bg=-1,fg=-1,italic=False,underline=False,strike=False,blinking=False,dim=False,hidden=False,inverse=False):
            settings = set_font(self.draw_line_bold, self.draw_line_bg, self.draw_line_fg)

            # top horizontal line
            tail = self.top_left_corner_chr
            body = self.top_horizontal_line_chr
            head = self.top_right_corner_chr
            print_horiz_sq_line(settings, self.adj_indent, length, tail, body, head)


            # left vertical line
            self.jumpTo(qty=1, direction=Move.UP)
            tail = self.top_left_corner_chr
            body = self.left_vertical_line_chr
            head = self.bottom_left_corner_chr
            print_vert_sq_line(settings, self.adj_indent, width, tail, body, head)


            # bottom horizontal line
            self.jumpTo(qty=1, direction=Move.UP)
            tail = self.bottom_left_corner_chr
            body = self.bottom_horizontal_line_chr
            head = self.bottom_right_corner_chr
            print_horiz_sq_line(settings, self.adj_indent, length, tail, body, head)


            # right vertical line
            self.jumpTo(qty=width,  direction=Move.UP)
            tail = self.top_right_corner_chr
            body = self.right_vertical_line_chr
            head = self.bottom_right_corner_chr
            print_vert_sq_line(settings, (length+self.adj_indent-1), width, tail, body, head)


            if style == Line_Style.CUSTOMIZED: pass
            else:
                # putting back all the default values
                # Horizontal Line Section
                self.top_horizontal_line_chr = thlc;    self.bottom_horizontal_line_chr = bhlc

                # Vertical Line Section
                self.left_vertical_line_chr = lvlc;     self.right_vertical_line_chr = rvlc

                # Corner Section
                self.top_left_corner_chr = tlcc;        self.top_right_corner_chr = trcc
                self.bottom_right_corner_chr = brcc;    self.bottom_left_corner_chr = blcc
            #-----------------------------------------------------------------------------------------------------------------------------------------



'''
custom_print module can handle any type of variable.
'''

#pylint: disable=bare-except
#pylint: disable=invalid-name
#pylint: disable=unused-import
#pylint: disable=line-too-long
#pylint: disable=too-many-lines
#pylint: disable=no-else-return
#pylint: disable=wildcard-import
#pylint: disable=unused-variable
#pylint: disable=too-many-locals
#pylint: disable=protected-access
#pylint: disable=too-many-branches
#pylint: disable=consider-using-in
#pylint: disable=chained-comparison
#pylint: disable=too-many-arguments
#pylint: disable=too-many-statements
#pylint: disable=multiple-statements
#pylint: disable=consider-using-join
#pylint: disable=unspecified-encoding
#pylint: disable=unnecessary-negation
#pylint: disable=singleton-comparison
#pylint: disable=too-few-public-methods
#pylint: disable=too-many-nested-blocks
#pylint: disable=too-many-public-methods
#pylint: disable=expression-not-assigned
#pylint: disable=consider-using-enumerate
#pylint: disable=unnecessary-comprehension
#pylint: disable=too-many-return-statements
#pylint: disable=unbalanced-tuple-unpacking
#pylint: disable=consider-using-max-builtin
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-positional-arguments
#pylint: disable=inconsistent-return-statements
#pylint: disable=possibly-used-before-assignment


#-----------------------------------------------------------------------------------------------------------------------------------------------------
#12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
#        1         2         3         4         5         6         7         8         9         A         B         C         D         E         F
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Required Modules                                                                                                                                   -
#-----------------------------------------------------------------------------------------------------------------------------------------------------
import enum
import platform        # fancy_functions
import csv             # PyLO class
import json            # PyLO class
import readline        # to use input and not cause problem with pylint
import os
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Layout is used for the Range, Set, Frozenset.                                                                                                      -
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#class Move(str, enum.Enum): # python3.9.18
class Move(enum.StrEnum):    # python3.12.1
# class Move():
    '''
    Move reference class
    '''
    UP    = "up"
    RIGHT = "right"
    DOWN  = "down"
    LEFT  = "left"


class Align(enum.StrEnum):
# class Align():
    '''
    Align reference class
    '''
    LEFT     = "left"
    CENTER   = "center"
    RIGHT    = "right"
    JUSTIFY  = "justify"


class Layout(enum.StrEnum):
# class Layout():
    '''
    Layout reference class
    '''
    HORIZONTAL = "horizontal"
    VERTICAL =   "vertical"


class Length_bg(enum.Enum):
# class Length_bg():
    '''
    Length reference class
    '''
    ALL_ROW   = 1
    ONLY_WORD = 2

class Divider_Style(enum.Enum):
    '''
    Divider Styles
    '''
    CUSTOMIZED   = "customized"
    SINGLE_LINE  = "single_line"
    SINGLE_BOLD  = "single_bold"
    SINGLE_HEAVY = "single_heavy"
    DOUBLE_LINE  = "double_line"
    DASH_1       = "dash_1"
    DASH_2       = "dash_2"
    SQ_BRACKETS  = "sq_brackets"
    BLUE_WHITE_1 = "blue_white_1"
    BLUE_WHITE_2 = "blue_white_2"



class Line_Style(enum.StrEnum):
# class Line_Style():
    '''
    Line_Style reference class
    '''
    CUSTOMIZED   = "customized"
    DASH         = "dash"
    DOUBLE_LINE  = "double_line"
    SINGLE_LINE  = "single_line"
    SINGLE_BOLD  = "single_bold"
    SINGLE_HEAVY = "single_heavy"
    SQ_BRACKETS  = "sq_brackets"

    # This two option are for making their own custom color for the user
    SINGLE_SPACE  = "single_space"
    DOUBLE_SPACE  = "double_space"
    NONE          = "none"
    # Color Designs ( DEF -> Default )
    # set_color_for_spaces_on_tbl(self, bg_color_line, bg_color_header, fg_color_header, bg_color_data, fg_color_data)
    # Design 1
    WHITE_PURPLE        = "white_purple"
    WHITE_BLACK_PURPLE  = "white_black_purple"
    RED_WHITE           = "red_white"
    PURPLE_WHITE        = "purple_white"
    BLUE_WHITE          = "blue_white"
    TURQUOISE_WHITE     = "turquoise_white"


    TEAL_WHITE      = "teal_white"
    GRAY_TEAL_WHITE = "gray_teal_white"
    BLUE_PURPLE_WHITE_1 = "blue_purple_white_1"
    BLUE_PURPLE_WHITE_2 = "blue_purple_white_2"
    GREEN_GREEN_BLACK   = "green_green_black"

    # set_color_2_for_tbl(self,bg_h, fg_h, bg_l, bg_d, fg_d)
    # Design 2
    WHITE_BLACK_1   = "white_black"
    WHITE_BLACK_2   = "white_black_v2"
    TURQUOISE_BLACK = "turquoise_black"

    DESIGN_1  = "design_1"
    DESIGN_2  = "design_2"
    DESIGN_3  = "design_3"
    DESIGN_4  = "design_4"
    DESIGN_5  = "design_5"
    DESIGN_6  = "design_6"
    DESIGN_7  = "design_7"
    DESIGN_8  = "design_8"
    DESIGN_9  = "design_9"
    DESIGN_10 = "design_10"

#-----------------------------------------------------------------------------------------------------------------------------------------------------
class Unicode(enum.StrEnum):
# class Unicode():
    '''
    Unicode reference class
    '''
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Box Drawings                                                                                                                                   -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    BOX_DRAWINGS_LIGHT_HORIZONTAL          = "\N{BOX DRAWINGS LIGHT HORIZONTAL}"
    BOX_DRAWINGS_LIGHT_VERTICAL_AND_RIGHT  = "\N{BOX DRAWINGS LIGHT VERTICAL AND RIGHT}"
    BOX_DRAWINGS_LIGHT_VERTICAL_AND_LEFT   = "\N{BOX DRAWINGS LIGHT VERTICAL AND LEFT}"

    BOX_DRAWINGS_LIGHT_VERTICAL            = "\N{BOX DRAWINGS LIGHT VERTICAL}"
    BOX_DRAWINGS_LIGHT_DOWN_AND_HORIZONTAL = "\N{BOX DRAWINGS LIGHT DOWN AND HORIZONTAL}"
    BOX_DRAWINGS_LIGHT_UP_AND_HORIZONTAL   = "\N{BOX DRAWINGS LIGHT UP AND HORIZONTAL}"

    BOX_DRAWINGS_LIGHT_VERTICAL_AND_HORIZONTAL ="\N{BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL}"

    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Triangle                                                                                                                                       -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    BLACK_UP_POINTING_TRIANGLE   = "\N{BLACK UP-POINTING TRIANGLE}"      # \u25B2  up fill arrow
    WHITE_UP_POINTING_TRIANGLE   = "\N{WHITE UP-POINTING TRIANGLE}"      # \u25B3  up empty arrow

    BLAKC_RIGHT_POINTING_TRIANGLE = "\N{BLACK RIGHT-POINTING TRIANGLE}"  # \u25B6  right fill  arrow
    WHITE_RIGHT_POINTING_TRIANGLE = "\N{WHITE RIGHT-POINTING TRIANGLE}"  # \u25B7  right empty arrow

    BLACK_DOWN_POINTING_TRIANGLE = "\N{BLACK DOWN-POINTING TRIANGLE}"    # \u25BC  down fill  arrow
    WHITE_DOWN_POINTING_TRIANGLE = "\N{BLACK DOWN-POINTING TRIANGLE}"    # \u25BD  down empty arrow

    BLACK_LEFT_POINTING_TRIANGLE = "\N{BLACK LEFT-POINTING TRIANGLE}"    # \u25C0  left fill arrow
    WHITE_LEFT_POINTING_TRIANGLE = "\N{WHITE LEFT-POINTING TRIANGLE}"    # \u25C1  left empty arrow

    EM_DASH = "\N{EM DASH}"
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Miscellaneous                                                                                                                                  -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    BLACK_DIAMOND = "\N{BLACK DIAMOND}"
    WHITE_DIAMOND = "\N{WHITE DIAMOND}"

    BLACK_CIRCLE  = "\N{BLACK CIRCLE}"
    WHITE_CIRCLE  = "\N{WHITE CIRCLE}"

    FACE = "(" + chr(0x25D5) + chr(0x25E1) + chr(0x25D5) + ")"

COLOR_NAMES = [\
"LIGHT BLACK           ",    "RED                   ",    "LIGHT OFFICE GREEN ",
"LIGHT BROWN           ",    "EARLY NIGHT BLUE      ",    "MID PURPLE         ",
"CYAN                  ",    "LIGHT GRAY            ",    "DARK GRAY          ",
"PASTEL RED            ",    "ELECTRIC LIGHT GREEN  ",    "DARKISH YELLOW     ",
"LIGHT BLUE            ",    "LIGHT PURPLE          ",    "VERY LIGHT BLUE    ",
"WHITE                 ",    "BLACK                 ",    "DARK BLUE          ",
"NAVY BLUE             ",    "MIDNIGHT BLUE         ",    "MEDDIUM BLUE       ",
"BLUE                  ",    "SUMMER GREEN          ",    "VERY DARK CYAN     ",
"SEA BLUE              ",    "ENDEAVOUR BLUE        ",    "SCIENCE BLUE       ",
"BLUE RIBBON           ",    "AO GREEN              ",    "DEEP SEA GREEN     ",
"TEAL                  ",    "DEEP CERULEAN BLUE    ",    "STRONG BLUE        ",
"AZURE BLUE            ",    "DARK LIME GREEN       ",    "GO GREEN           ",
"DARK CYAN             ",    "BONDI BLUE            ",    "CERULEAN BLUE      ",
"BLUE BOLT             ",    "STRONG LIME GREEN     ",    "MALACHITE GREEN    ",
"CARIBBEAN GREEN       ",    "STRONG CYAN           ",    "DARK TURQUOISE     ",
"VIVID SKY BLUE        ",    "ELECTRIC GREEN        ",    "SPRING GREEN       ",
"GUPPIE GREEN          ",    "MEDIUM SPRING GREEN   ",    "BRIGHT TURQUOISE   ",
"AQUA                  ",    "BLOOD RED             ",    "VERY DARK MAGENTA  ",
"INDIGO                ",    "DARK VIOLET           ",    "LIGHT_VIOLET       ",
"ELECTRIC INDIGO       ",    "VERDUN GREEN          ",    "SCORPION GRAY      ",
"UCLA BLUE             ",    "SCAMPI BLUE           ",    "SLATE BLUE         ",
"CORNFLOWER BLUE       ",    "AVOCADO GREEN         ",    "GLADE GREEN        ",
"STEEL TEAL CYAN       ",    "STEEL BLUE            ",    "HAVELOCK BLUE      ",
"BLUEBERRY             ",    "KELLY GREEN           ",    "FOREST GREEN       ",
"POLISHED PIN GREEN    ",    "CRYSTAL BLUE          ",    "AQUA PEARL         ",
"BLUE JEANS            ",    "HARLEQUIN GREEN       ",    "MODERATE LIME GREEN",
"CARIBBEAN GREEN PEARL ",    "EUCALYPTUS GREEN      ",    "MEDDIUM TURQUOISE  ",
"MAYA BLUE             ",    "BRIGHT GREEN          ",    "LIGHT LIME GREEN   ",
"LIGHT MALACHITE GREEN ",    "MEDDIUM AQUAMARINE    ",    "AQUAMARINE GREEN   ",
"AQUAMARINE CYAN       ",    "DEEP RED              ",    "FRENCH PLUM VIOLET ",
"FRESH EGGPLANT VIOLET ",    "VIOLET                ",    "STRONG VIOLET      ",
"ELECTRIC VIOLET       ",    "BROWN                 ",    "COPPER BROWN       ",
"MOSTLY VIOLET         ",    "ROYAL PURPLE          ",    "MEDDIUM PURPLE     ",
"BLUEBERRY PURPLE      ",    "DARK OLIVE GREEN      ",    "CLAY CREEK GREEN   ",
"TAUPE GRAY            ",    "COOL GRAY             ",    "CHETWODE BLUE      ",
"VIOLET BLUE           ",    "APPLE GREEN           ",    "ASPARAGUS GREEN    ",
"LEAF GREEN            ",    "GRAYISH CYAN          ",    "COBALT BLUE        ",
"SKY BLUE              ",    "PISTACHIO GREEN       ",    "MANTIS GREEN       ",
"PASTEL GREEN          ",    "PEARL AQUA            ",    "SLIGHTLY CYAN      ",
"PALE CYAN             ",    "GREEN                 ",    "LIGHT GREEN        ",
"VERY LIGHT LIME GREEN ",    "MINT GREEN            ",    "AQUA LIME GREEN    ",
"LIGHT CYAN            ",    "DARK RED              ",    "DARK PINK          ",
"DARK MAGENTA          ",    "HELIOTROPE MAGENTA    ",    "VIVID PURPLE       ",
"ELECTRIC PURPLE       ",    "DARK ORANGE BROWN     ",    "ELECTRIC BROWN     ",
"DARK MODERATE PINK    ",    "DARK MODERATE MAGENTA ",    "RICH LILAC VIOLET  ",
"LAVENDER INDIGO       ",    "PIRATE GOLD BROWN     ",    "BRONZE BROWN       ",
"DARK GRAYISH RED      ",    "DARK GRAYISH MAGENTA  ",    "LAVENDER           ",
"BRIGHT LAVENDER       ",    "LIGHT GOLD BROWN      ",    "LIGHT OLIVE GREEN  ",
"DARK GRAYISH YELLOW   ",    "SILVER FOIL           ",    "GRAYISH BLUE       ",
"BLUE PURPLE           ",    "VIVID LIME GREEN      ",    "MODERATE GREEN     ",
"YELLOW GREEN          ",    "GRAYISH LIME GREEN    ",    "CRYSTAL CYAN       ",
"PALE BLUE             ",    "LIME                  ",    "GREEN YELLOW       ",
"VERY LIGHT GREEN      ",    "MENTHOL GREEN         ",    "AEREO BLUE         ",
"CELESTE CYAN          ",    "STRONG RED            ",    "ROYAL RED          ",
"MEXICAN PINK          ",    "HOLLYWOOD PINK        ",    "STRONG MAGENTA     ",
"PHLOX VIOLET          ",    "STRONG ORANGE         ",    "INDIAN RED         ",
"BLUSH RED             ",    "SUPER PINK            ",    "ORCHID MAGENTA     ",
"LIGHT MAGENTA         ",    "CHOCOLATE BROWN       ",    "COPPERFIELD BROWN  ",
"SLIGHTLY RED          ",    "SLIGHTLY PINK         ",    "LIGHT ORCHID PINK  ",
"BRIGHT LILAC VIOLET   ",    "MUSTARD YELLOW        ",    "EARTH YELLOW       ",
"TAN BROWN             ",    "GRAYISH RED           ",    "GRAYISH MAGENTA    ",
"PALE VIOLET           ",    "STRONG YELLOW         ",    "MODERATE YELLOW    ",
"DECO YELLOW           ",    "PASTEL GRAY           ",    "LIGHT SILVER       ",
"PALE LAVENDER         ",    "NEON YELLOW           ",    "LIGHT GREEN YELLOW ",
"MINDARO YELLOW        ",    "PALE GREEN            ",    "VERY PALE GREEN    ",
"VERY LIGHT CYAN       ",    "LIGHT RED             ",    "RASPBERRY RED      ",
"BRIGHT PINK           ",    "PINK                  ",    "MAGENTA            ",
"FUCHSIA               ",    "BLAZE ORANGE          ",    "BITTERSWEET RED    ",
"STRAWBERRY RED        ",    "HOT PINK              ",    "LIGHT PINK         ",
"PINK FLAMINGO         ",    "DARK ORANGE           ",    "SALMON ORANGE      ",
"TANGERINE RED         ",    "PINK SALMON           ",    "LAVENDER ROSE      ",
"FUCHSIA PINK          ",    "ORANGE                ",    "LIGHT ORANGE       ",
"VERY LIGHT ORANGE     ",    "PALE RED              ",    "PALE PINK          ",
"PALE MAGENTA          ",    "GOLD                  ",    "DANDELION YELLOW   ",
"JASMINE BROWN         ",    "PALE ORANGE           ",    "MISTY ROSE PINK    ",
"PINK LACE             ",    "YELLOW                ",    "LEMON YELLOW       ",
"PASTEL YELLOW         ",    "PALE YELLOW           ",    "VERY PALE YELLOW   ",
"LIGHT WHITE           ",    "VAMPIRE BLACK         ",    "GRAY BLACK         ",
"EERIE BLACK           ",    "RAISIN BLACK          ",    "DARK CHARCOAL      ",
"BLACK OLIVE           ",    "OUTER SPACE GRAY      ",    "DARK LIVER GRAY    ",
"DAVYS GRAY            ",    "GRANITE GRAY          ",    "DIM GRAY           ",
"SONIC SILVER          ",    "GRAY                  ",    "PHILIPPINE GRAY    ",
"DUSTY GRAY            ",    "SPANISH GRAY          ",    "LIGHTISH GRAY      ",
"PHILIPPINE SILVER     ",    "SILVER                ",    "SILVER SAND.       ",
"AMERICAN SILVER       ",    "ALTO GRAY             ",    "MERCURY GRAY       ",
"DARK WHITE            ",    "DEFAULT               ",    "DEFAULT            "]



class No(enum.IntEnum):
    ''' Color class will help to select a specific color by name rather than using the number.
    The number can be known by using the methods bg_ansi_color or fg_ansi_color.

    Notice that Color class works with all the classes, methods, and functions from
    custom_print rather than itself.

    import custom_print as cp

    CORRECT -> print(f"{cp.set_font(True, cp.Color.SUMMER_GREEN, cp.Color.BLACK)} Hello There...! {cp.reset_font()}")

    WRONG ->print(f"{cp.Color.SUMMER_GREEN} Hello There...! {cp.reset_font}")
    For the lates option use the Bg and Fg and Style classes.
        '''

    LIGHT_BLACK           = 0;          RED                   = 1;          LIGHT_OFFICE_GREEN  = 2
    LIGHT_BROWN           = 3;          EARLY_NIGHT_BLUE      = 4;          MED_PURPLE          = 5
    CYAN                  = 6;          LIGHT_GRAY            = 7;          DARK_GRAY           = 8
    PASTEL_RED            = 9;          ELECTRIC_LIGHT_GREEN  = 10;         DARKISH_YELLOW      = 11
    LIGHT_BLUE            = 12;         LIGHT_PURPLE          = 13;         VERY_LIGHT_BLUE     = 14
    WHITE                 = 15;         BLACK                 = 16;         DARK_BLUE           = 17
    NAVY_BLUE             = 18;         MIDNIGHT_BLUE         = 19;         MEDDIUM_BLUE        = 20
    BLUE                  = 21;         SUMMER_GREEN          = 22;         VERY_DARK_CYAN      = 23
    SEA_BLUE              = 24;         ENDEAVOUR_BLUE        = 25;         SCIENCE_BLUE        = 26
    BLUE_RIBBON           = 27;         AO_GREEN              = 28;         DEEP_SEA_GREEN      = 29
    TEAL                  = 30;         DEEP_CERULEAN_BLUE    = 31;         STRONG_BLUE         = 32
    AZURE_BLUE            = 33;         DARK_LIME_GREEN       = 34;         GO_GREEN            = 35
    DARK_CYAN             = 36;         BONDI_BLUE            = 37;         CERULEAN_BLUE       = 38
    BLUE_BOLT             = 39;         STRONG_LIME_GREEN     = 40;         MALACHITE_GREEN     = 41
    CARIBBEAN_GREEN       = 42;         STRONG_CYAN           = 43;         DARK_TURQUOISE      = 44
    VIVID_SKY_BLUE        = 45;         ELECTRIC_GREEN        = 46;         SPRING_GREEN        = 47
    GUPPIE_GREEN          = 48;         MEDIUM_SPRING_GREEN   = 49;         BRIGHT_TURQUOISE    = 50
    AQUA                  = 51;         BLOOD_RED             = 52;         VERY_DARK_MAGENTA   = 53
    INDIGO                = 54;         DARK_VIOLET           = 55;         LIGHT_VIOLET        = 56
    ELECTRIC_INDIGO       = 57;         VERDUN_GREEN          = 58;         SCORPION_GRAY       = 59
    UCLA_BLUE             = 60;         SCAMPI_BLUE           = 61;         SLATE_BLUE          = 62
    CORNFLOWER_BLUE       = 63;         AVOCADO_GREEN         = 64;         GLADE_GREEN         = 65
    STEEL_TEAL_CYAN       = 66;         STEEL_BLUE            = 67;         HAVELOCK_BLUE       = 68
    BLUEBERRY             = 69;         KELLY_GREEN           = 70;         FOREST_GREEN        = 71
    POLISHED_PIN_GREEN    = 72;         CRYSTAL_BLUE          = 73;         AQUA_PEARL          = 74
    BLUE_JEANS            = 75;         HARLEQUIN_GREEN       = 76;         MODERATE_LIME_GREEN = 77
    CARIBBEAN_GREEN_PEARL = 78;         EUCALYPTUS_GREEN      = 79;         MEDDIUM_TURQUOISE   = 80
    MAYA_BLUE             = 81;         BRIGHT_GREEN          = 82;         LIGHT_LIME_GREEN    = 83
    LIGHT_MALACHITE_GREEN = 84;         MEDDIUM_AQUAMARINE    = 85;         AQUAMARINE_GREEN    = 86
    AQUAMARINE_CYAN       = 87;         DEEP_RED              = 88;         FRENCH_PLUM_VIOLET  = 89
    FRESH_EGGPLANT_VIOLET = 90;         VIOLET                = 91;         STRONG_VIOLET       = 92
    ELECTRIC_VIOLET       = 93;         BROWN                 = 94;         COPPER_BROWN        = 95
    MOSTLY_VIOLET         = 96;         ROYAL_PURPLE          = 97;         MEDDIUM_PURPLE      = 98
    BLUEBERRY_PURPLE      = 99;         DARK_OLIVE_GREEN      = 100;        CLAY_CREEK_GREEN    = 101
    TAUPE_GRAY            = 102;        COOL_GRAY             = 103;        CHETWODE_BLUE       = 104
    VIOLET_BLUE           = 105;        APPLE_GREEN           = 106;        ASPARAGUS_GREEN     = 107
    LEAF_GREEN            = 108;        GRAYISH_CYAN          = 109;        COBALT_BLUE         = 110
    SKY_BLUE              = 111;        PISTACHIO_GREEN       = 112;        MANTIS_GREEN        = 113
    PASTEL_GREEN          = 114;        PEARL_AQUA            = 115;        SLIGHTLY_CYAN       = 116
    PALE_CYAN             = 117;        GREEN                 = 118;        LIGHT_GREEN         = 119
    VERY_LIGHT_LIME_GREEN = 120;        MINT_GREEN            = 121;        AQUA_LIME_CYAN      = 122
    LIGHT_CYAN            = 123;        DARK_RED              = 124;        DARK_PINK           = 125
    DARK_MAGENTA          = 126;        HELIOTROPE_MAGENTA    = 127;        VIVID_PURPLE        = 128
    ELECTRIC_PURPLE       = 129;        DARK_ORANGE_BROWN     = 130;        ELECTRIC_BROWN      = 131
    DARK_MODERATE_PINK    = 132;        DARK_MODERATE_MAGENTA = 133;        RICH_LILAC_VIOLET   = 134
    LAVENDER_INDIGO       = 135;        PIRATE_GOLD_BROWN     = 136;        BRONZE_BROWN        = 137
    DARK_GRAYISH_RED      = 138;        DARK_GRAYISH_MAGENTA  = 139;        LAVENDER            = 140
    BRIGHT_LAVENDER       = 141;        LIGHT_GOLD_BROWN      = 142;        LIGHT_OLIVE_GREEN   = 143
    DARK_GRAYISH_YELLOW   = 144;        SILVER_FOIL           = 145;        GRAYISH_BLUE        = 146
    BLUE_PURPLE           = 147;        VIVID_LIME_GREEN      = 148;        MODERATE_GREEN      = 149
    YELLOW_GREEN          = 150;        GRAYISH_LIME_GREEN    = 151;        CRYSTAL_CYAN        = 152
    PALE_BLUE             = 153;        LIME                  = 154;        GREEN_YELLOW        = 155
    VERY_LIGHT_GREEN      = 156;        MENTHOL_GREEN         = 157;        AEREO_BLUE          = 158
    CELESTE_CYAN          = 159;        STRONG_RED            = 160;        ROYAL_RED           = 161
    MEXICAN_PINK          = 162;        HOLLYWOOD_PINK        = 163;        STRONG_MAGENTA      = 164
    PHLOX_VIOLET          = 165;        STRONG_ORANGE         = 166;        INDIAN_RED          = 167
    BLUSH_RED             = 168;        SUPER_PINK            = 169;        ORCHID_MAGENTA      = 170
    LIGHT_MAGENTA         = 171;        CHOCOLATE_BROWN       = 172;        COPPERFIELD_BROWN   = 173
    SLIGHTLY_RED          = 174;        SLIGHTLY_PINK         = 175;        LIGHT_ORCHID_PINK   = 176
    BRIGHT_LILAC_VIOLET   = 177;        MUSTARD_YELLOW        = 178;        EARTH_YELLOW        = 179
    TAN_BROWN             = 180;        GRAYISH_RED           = 181;        GRAYISH_MAGENTA     = 182
    PALE_VIOLET           = 183;        STRONG_YELLOW         = 184;        MODERATE_YELLOW     = 185
    DECO_YELLOW           = 186;        PASTEL_GRAY           = 187;        LIGHT_SILVER        = 188
    PALE_LAVENDER         = 189;        NEON_YELLOW           = 190;        LIGHT_GREEN_YELLOW  = 191
    MINDARO_YELLOW        = 192;        PALE_GREEN            = 193;        VERY_PALE_GREEN     = 194
    VERY_LIGHT_CYAN       = 195;        LIGHT_RED             = 196;        RASPBERRY_RED       = 197
    BRIGHT_PINK           = 198;        PINK                  = 199;        MAGENTA             = 200
    FUCHSIA               = 201;        BLAZE_ORANGE          = 202;        BITTERSWEET_RED     = 203
    STRAWBERRY_RED        = 204;        HOT_PINK              = 205;        LIGHT_PINK          = 206
    PINK_FLAMINGO         = 207;        DARK_ORANGE           = 208;        SALMON_ORANGE       = 209
    TANGERINE_RED         = 210;        PINK_SALMON           = 211;        LAVENDER_ROSE       = 212
    FUCHSIA_PINK          = 213;        ORANGE                = 214;        LIGHT_ORANGE        = 215
    VERY_LIGHT_ORANGE     = 216;        PALE_RED              = 217;        PALE_PINK           = 218
    PALE_MAGENTA          = 219;        GOLD                  = 220;        DANDELION_YELLOW    = 221
    JASMINE_BROWN         = 222;        PALE_ORANGE           = 223;        MISTY_ROSE_PINK     = 224
    PINK_LACE             = 225;        YELLOW                = 226;        LEMON_YELLOW        = 227
    PASTEL_YELLOW         = 228;        PALE_YELLOW           = 229;        VERY_PALE_YELLOW    = 230
    LIGHT_WHITE           = 231;        VAMPIRE_BLACK         = 232;        GRAY_BLACK          = 233
    EERIE_BLACK           = 234;        RAISIN_BLACK          = 235;        DARK_CHARCOAL       = 236
    BLACK_OLIVE           = 237;        OUTER_SPACE_GRAY      = 238;        DARK_LIVER_GRAY     = 239
    DAVYS_GRAY            = 240;        GRANITE_GRAY          = 241;        DIM_GRAY            = 242
    SONIC_SILVER          = 243;        GRAY                  = 244;        PHILIPPINE_GRAY     = 245
    DUSTY_GRAY            = 246;        SPANISH_GRAY          = 247;        LIGHTISH_GRAY       = 248
    PHILIPPINE_SILVER     = 249;        SILVER                = 250;        SILVER_SAND         = 251
    AMERICAN_SILVER       = 252;        ALTO_GRAY             = 253;        MERCURY_GRAY        = 254
    DARK_WHITE            = 255;        DEFAULT               = 256;        default             = -1

class Bg(enum.StrEnum):

    '''  This Class uses the name of the color for background.
         import custom_print as cp
         print(f"{cp.Bg.INDIGO} Hello {cp.Bg.OFF} Normal")
    '''

    LIGHT_BLACK           = "\033[48;5;0m";          RED                    = "\033[48;5;1m";         LIGHT_OFFICE_GREEN   = "\033[48;5;2m"
    LIGHT_BROWN           = "\033[48;5;3m";          EARLY_NIGHT_BLUE       = "\033[48;5;4m";         MED_PURPLE           = "\033[48;5;5m"
    CYAN                  = "\033[48;5;6m";          LIGHT_GRAY             = "\033[48;5;7m";         DARK_GRAY            = "\033[48;5;8m"
    PASTEL_RED            = "\033[48;5;9m";          ELECTRIC_LIGHT_GREEN   = "\033[48;5;10m";        DARKISH_YELLOW       = "\033[48;5;11m"
    LIGHT_BLUE            = "\033[48;5;12m";         LIGHT_PURPLE           = "\033[48;5;13m";         VERY_LIGHT_BLUE     = "\033[48;5;14m"
    WHITE                 = "\033[48;5;15m";         BLACK                  = "\033[48;5;16m";         DARK_BLUE           = "\033[48;5;17m"
    NAVY_BLUE             = "\033[48;5;18m";         MIDNIGHT_BLUE          = "\033[48;5;19m";         MEDDIUM_BLUE        = "\033[48;5;20m"
    BLUE                  = "\033[48;5;21m";         SUMMER_GREEN           = "\033[48;5;22m";         VERY_DARK_CYAN      = "\033[48;5;23m"
    SEA_BLUE              = "\033[48;5;24m";         ENDEAVOUR_BLUE         = "\033[48;5;25m";         SCIENCE_BLUE        = "\033[48;5;26m"
    BLUE_RIBBON           = "\033[48;5;27m";         AO_GREEN               = "\033[48;5;28m";         DEEP_SEA_GREEN      = "\033[48;5;29m"
    TEAL                  = "\033[48;5;30m";         DEEP_CERULEAN_BLUE     = "\033[48;5;31m";         STRONG_BLUE         = "\033[48;5;32m"
    AZURE_BLUE            = "\033[48;5;33m";         DARK_LIME_GREEN        = "\033[48;5;34m";         GO_GREEN            = "\033[48;5;35m"
    DARK_CYAN             = "\033[48;5;36m";         BONDI_BLUE             = "\033[48;5;37m";         CERULEAN_BLUE       = "\033[48;5;38m"
    BLUE_BOLT             = "\033[48;5;39m";         STRONG_LIME_GREEN      = "\033[48;5;40m";         MALACHITE_GREEN     = "\033[48;5;41m"
    CARIBBEAN_GREEN       = "\033[48;5;42m";         STRONG_CYAN            = "\033[48;5;43m";         DARK_TURQUOISE      = "\033[48;5;44m"
    VIVID_SKY_BLUE        = "\033[48;5;45m";         ELECTRIC_GREEN         = "\033[48;5;46m";         SPRING_GREEN        = "\033[48;5;47m"
    GUPPIE_GREEN          = "\033[48;5;48m";         MEDIUM_SPRING_GREEN    = "\033[48;5;49m";         BRIGHT_TURQUOISE    = "\033[48;5;50m"
    AQUA                  = "\033[48;5;51m";         BLOOD_RED              = "\033[48;5;52m";         VERY_DARK_MAGENTA   = "\033[48;5;53m"
    INDIGO                = "\033[48;5;54m";         DARK_VIOLET            = "\033[48;5;55m";         LIGHT_VIOLET        = "\033[48;5;56m"
    ELECTRIC_INDIGO       = "\033[48;5;57m";         VERDUN_GREEN           = "\033[48;5;58m";         SCORPION_GRAY       = "\033[48;5;59m"
    UCLA_BLUE             = "\033[48;5;60m";         SCAMPI_BLUE            = "\033[48;5;61m";         SLATE_BLUE          = "\033[48;5;62m"
    CORNFLOWER_BLUE       = "\033[48;5;63m";         AVOCADO_GREEN          = "\033[48;5;64m";         GLADE_GREEN         = "\033[48;5;65m"
    STEEL_TEAL_CYAN       = "\033[48;5;66m";         STEEL_BLUE             = "\033[48;5;67m";         HAVELOCK_BLUE       = "\033[48;5;68m"
    BLUEBERRY             = "\033[48;5;69m";         KELLY_GREEN            = "\033[48;5;70m";         FOREST_GREEN        = "\033[48;5;71m"
    POLISHED_PIN_GREEN    = "\033[48;5;72m";         CRYSTAL_BLUE           = "\033[48;5;73m";         AQUA_PEARL          = "\033[48;5;74m"
    BLUE_JEANS            = "\033[48;5;75m";         HARLEQUIN_GREEN        = "\033[48;5;76m";         MODERATE_LIME_GREEN = "\033[48;5;77m"
    CARIBBEAN_GREEN_PEARL = "\033[48;5;78m";         EUCALYPTUS_GREEN       = "\033[48;5;79m";         MEDDIUM_TURQUOISE   = "\033[48;5;80m"
    MAYA_BLUE             = "\033[48;5;81m";         BRIGHT_GREEN           = "\033[48;5;82m";         LIGHT_LIME_GREEN    = "\033[48;5;83m"
    LIGHT_MALACHITE_GREEN = "\033[48;5;84m";         MEDDIUM_AQUAMARINE     = "\033[48;5;85m";         AQUAMARINE_GREEN    = "\033[48;5;86m"
    AQUAMARINE_CYAN       = "\033[48;5;87m";         DEEP_RED               = "\033[48;5;88m";         FRENCH_PLUM_VIOLET  = "\033[48;5;89m"
    FRESH_EGGPLANT_VIOLET = "\033[48;5;90m";         VIOLET                 = "\033[48;5;91m";         STRONG_VIOLET       = "\033[48;5;92m"
    ELECTRIC_VIOLET       = "\033[48;5;93m";         BROWN                  = "\033[48;5;94m";         COPPER_BROWN        = "\033[48;5;95m"
    MOSTLY_VIOLET         = "\033[48;5;96m";         ROYAL_PURPLE           = "\033[48;5;97m";         MEDDIUM_PURPLE      = "\033[48;5;98m"
    BLUEBERRY_PURPLE      = "\033[48;5;99m";         DARK_OLIVE_GREEN       = "\033[48;5;100m";        CLAY_CREEK_GREEN    = "\033[48;5;101m"
    TAUPE_GRAY            = "\033[48;5;102m";        COOL_GRAY              = "\033[48;5;103m";        CHETWODE_BLUE       = "\033[48;5;104m"
    VIOLET_BLUE           = "\033[48;5;105m";        APPLE_GREEN            = "\033[48;5;106m";        ASPARAGUS_GREEN     = "\033[48;5;107m"
    LEAF_GREEN            = "\033[48;5;108m";        GRAYISH_CYAN           = "\033[48;5;109m";        COBALT_BLUE         = "\033[48;5;110m"
    SKY_BLUE              = "\033[48;5;111m";        PISTACHIO_GREEN        = "\033[48;5;112m";        MANTIS_GREEN        = "\033[48;5;113m"
    PASTEL_GREEN          = "\033[48;5;114m";        PEARL_AQUA             = "\033[48;5;115m";        SLIGHTLY_CYAN       = "\033[48;5;116m"
    PALE_CYAN             = "\033[48;5;117m";        GREEN                  = "\033[48;5;118m";        LIGHT_GREEN         = "\033[48;5;119m"
    VERY_LIGHT_LIME_GREEN = "\033[48;5;120m";        MINT_GREEN             = "\033[48;5;121m";        AQUA_LIME_CYAN      = "\033[48;5;122m"
    LIGHT_CYAN            = "\033[48;5;123m";        DARK_RED               = "\033[48;5;124m";        DARK_PINK           = "\033[48;5;125m"
    DARK_MAGENTA          = "\033[48;5;126m";        HELIOTROPE_MAGENTA     = "\033[48;5;127m";        VIVID_PURPLE        = "\033[48;5;128m"
    ELECTRIC_PURPLE       = "\033[48;5;129m";        DARK_ORANGE_BROWN      = "\033[48;5;130m";        ELECTRIC_BROWN      = "\033[48;5;131m"
    DARK_MODERATE_PINK    = "\033[48;5;132m";        DARK_MODERATE_MAGENTA  = "\033[48;5;133m";        RICH_LILAC_VIOLET   = "\033[48;5;134m"
    LAVENDER_INDIGO       = "\033[48;5;135m";        PIRATE_GOLD_BROWN      = "\033[48;5;136m";        BRONZE_BROWN        = "\033[48;5;137m"
    DARK_GRAYISH_RED      = "\033[48;5;138m";        DARK_GRAYISH_MAGENTA   = "\033[48;5;139m";        LAVENDER            = "\033[48;5;140m"
    BRIGHT_LAVENDER       = "\033[48;5;141m";        LIGHT_GOLD_BROWN       = "\033[48;5;142m";        LIGHT_OLIVE_GREEN   = "\033[48;5;143m"
    DARK_GRAYISH_YELLOW   = "\033[48;5;144m";        SILVER_FOIL            = "\033[48;5;145m";        GRAYISH_BLUE        = "\033[48;5;146m"
    BLUE_PURPLE           = "\033[48;5;147m";        VIVID_LIME_GREEN       = "\033[48;5;148m";        MODERATE_GREEN      = "\033[48;5;149m"
    YELLOW_GREEN          = "\033[48;5;150m";        GRAYISH_LIME_GREEN     = "\033[48;5;151m";        CRYSTAL_CYAN        = "\033[48;5;152m"
    PALE_BLUE             = "\033[48;5;153m";        LIME                   = "\033[48;5;154m";        GREEN_YELLOW        = "\033[48;5;155m"
    VERY_LIGHT_GREEN      = "\033[48;5;156m";        MENTHOL_GREEN          = "\033[48;5;157m";        AEREO_BLUE          = "\033[48;5;158m"
    CELESTE_CYAN          = "\033[48;5;159m";        STRONG_RED             = "\033[48;5;160m";        ROYAL_RED           = "\033[48;5;161m"
    MEXICAN_PINK          = "\033[48;5;162m";        HOLLYWOOD_PINK         = "\033[48;5;163m";        STRONG_MAGENTA      = "\033[48;5;164m"
    PHLOX_VIOLET          = "\033[48;5;165m";        STRONG_ORANGE          = "\033[48;5;166m";        INDIAN_RED          = "\033[48;5;167m"
    BLUSH_RED             = "\033[48;5;168m";        SUPER_PINK             = "\033[48;5;169m";        ORCHID_MAGENTA      = "\033[48;5;170m"
    LIGHT_MAGENTA         = "\033[48;5;171m";        CHOCOLATE_BROWN        = "\033[48;5;172m";        COPPERFIELD_BROWN   = "\033[48;5;173m"
    SLIGHTLY_RED          = "\033[48;5;174m";        SLIGHTLY_PINK          = "\033[48;5;175m";        LIGHT_ORCHID_PINK   = "\033[48;5;176m"
    BRIGHT_LILAC_VIOLET   = "\033[48;5;177m";        MUSTARD_YELLOW         = "\033[48;5;178m";        EARTH_YELLOW        = "\033[48;5;179m"
    TAN_BROWN             = "\033[48;5;180m";        GRAYISH_RED            = "\033[48;5;181m";        GRAYISH_MAGENTA     = "\033[48;5;182m"
    PALE_VIOLET           = "\033[48;5;183m";        STRONG_YELLOW          = "\033[48;5;184m";        MODERATE_YELLOW     = "\033[48;5;185m"
    DECO_YELLOW           = "\033[48;5;186m";        PASTEL_GRAY            = "\033[48;5;187m";        LIGHT_SILVER        = "\033[48;5;188m"
    PALE_LAVENDER         = "\033[48;5;189m";        NEON_YELLOW            = "\033[48;5;190m";        LIGHT_GREEN_YELLOW  = "\033[48;5;191m"
    MINDARO_YELLOW        = "\033[48;5;192m";        PALE_GREEN             = "\033[48;5;193m";        VERY_PALE_GREEN     = "\033[48;5;194m"
    VERY_LIGHT_CYAN       = "\033[48;5;195m";        LIGHT_RED              = "\033[48;5;196m";        RASPBERRY_RED       = "\033[48;5;197m"
    BRIGHT_PINK           = "\033[48;5;198m";        PINK                   = "\033[48;5;199m";        MAGENTA             = "\033[48;5;200m"
    FUCHSIA               = "\033[48;5;201m";        BLAZE_ORANGE           = "\033[48;5;202m";        BITTERSWEET_RED     = "\033[48;5;203m"
    STRAWBERRY_RED        = "\033[48;5;204m";        HOT_PINK               = "\033[48;5;205m";        LIGHT_PINK          = "\033[48;5;206m"
    PINK_FLAMINGO         = "\033[48;5;207m";        DARK_ORANGE            = "\033[48;5;208m";        SALMON_ORANGE       = "\033[48;5;209m"
    TANGERINE_RED         = "\033[48;5;210m";        PINK_SALMON            = "\033[48;5;211m";        LAVENDER_ROSE       = "\033[48;5;212m"
    FUCHSIA_PINK          = "\033[48;5;213m";        ORANGE                 = "\033[48;5;214m";        LIGHT_ORANGE        = "\033[48;5;215m"
    VERY_LIGHT_ORANGE     = "\033[48;5;216m";        PALE_RED               = "\033[48;5;217m";        PALE_PINK           = "\033[48;5;218m"
    PALE_MAGENTA          = "\033[48;5;219m";        GOLD                   = "\033[48;5;220m";        DANDELION_YELLOW    = "\033[48;5;221m"
    JASMINE_BROWN         = "\033[48;5;222m";        PALE_ORANGE            = "\033[48;5;223m";        MISTY_ROSE_PINK     = "\033[48;5;224m"
    PINK_LACE             = "\033[48;5;225m";        YELLOW                 = "\033[48;5;226m";        LEMON_YELLOW        = "\033[48;5;227m"
    PASTEL_YELLOW         = "\033[48;5;228m";        PALE_YELLOW            = "\033[48;5;229m";        VERY_PALE_YELLOW    = "\033[48;5;230m"
    LIGHT_WHITE           = "\033[48;5;231m";        VAMPIRE_BLACK          = "\033[48;5;232m";        GRAY_BLACK          = "\033[48;5;233m"
    EERIE_BLACK           = "\033[48;5;234m";        RAISIN_BLACK           = "\033[48;5;235m";        DARK_CHARCOAL       = "\033[48;5;236m"
    BLACK_OLIVE           = "\033[48;5;237m";        OUTER_SPACE_GRAY       = "\033[48;5;238m";        DARK_LIVER_GRAY     = "\033[48;5;239m"
    DAVYS_GRAY            = "\033[48;5;240m";        GRANITE_GRAY           = "\033[48;5;241m";        DIM_GRAY            = "\033[48;5;242m"
    SONIC_SILVER          = "\033[48;5;243m";        GRAY                   = "\033[48;5;244m";        PHILIPPINE_GRAY     = "\033[48;5;245m"
    DUSTY_GRAY            = "\033[48;5;246m";        SPANISH_GRAY           = "\033[48;5;247m";        LIGHTISH_GRAY       = "\033[48;5;248m"
    PHILIPPINE_SILVER     = "\033[48;5;249m";        SILVER                 = "\033[48;5;250m";        SILVER_SAND         = "\033[48;5;251m"
    AMERICAN_SILVER       = "\033[48;5;252m";        ALTO_GRAY              = "\033[48;5;253m";        MERCURY_GRAY        = "\033[48;5;254m"
    DARK_WHITE            = "\033[48;5;255m";        DEFAULT                = "\033[49m";              OFF                 = "\033[49m"





class Fg(enum.StrEnum):

    '''  This Class uses the name of the color for the foreground.
         import custom_print as cp
         print(f"{cp.Fg.YELLOW} Hello {cp.Fg.OFF} Normal")
    '''

    LIGHT_BLACK           = "\033[38;5;0m";          RED                    = "\033[38;5;1m";         LIGHT_OFFICE_GREEN   = "\033[38;5;2m"
    LIGHT_BROWN           = "\033[38;5;3m";          EARLY_NIGHT_BLUE       = "\033[38;5;4m";         MED_PURPLE           = "\033[38;5;5m"
    CYAN                  = "\033[38;5;6m";          LIGHT_GRAY             = "\033[38;5;7m";         DARK_GRAY            = "\033[38;5;8m"
    PASTEL_RED            = "\033[38;5;9m";          ELECTRIC_LIGHT_GREEN   = "\033[38;5;10m";        DARKISH_YELLOW       = "\033[38;5;11m"
    LIGHT_BLUE            = "\033[38;5;12m";         LIGHT_PURPLE           = "\033[38;5;13m";         VERY_LIGHT_BLUE     = "\033[38;5;14m"
    WHITE                 = "\033[38;5;15m";         BLACK                  = "\033[38;5;16m";         DARK_BLUE           = "\033[38;5;17m"
    NAVY_BLUE             = "\033[38;5;18m";         MIDNIGHT_BLUE          = "\033[38;5;19m";         MEDDIUM_BLUE        = "\033[38;5;20m"
    BLUE                  = "\033[38;5;21m";         SUMMER_GREEN           = "\033[38;5;22m";         VERY_DARK_CYAN      = "\033[38;5;23m"
    SEA_BLUE              = "\033[38;5;24m";         ENDEAVOUR_BLUE         = "\033[38;5;25m";         SCIENCE_BLUE        = "\033[38;5;26m"
    BLUE_RIBBON           = "\033[38;5;27m";         AO_GREEN               = "\033[38;5;28m";         DEEP_SEA_GREEN      = "\033[38;5;29m"
    TEAL                  = "\033[38;5;30m";         DEEP_CERULEAN_BLUE     = "\033[38;5;31m";         STRONG_BLUE         = "\033[38;5;32m"
    AZURE_BLUE            = "\033[38;5;33m";         DARK_LIME_GREEN        = "\033[38;5;34m";         GO_GREEN            = "\033[38;5;35m"
    DARK_CYAN             = "\033[38;5;36m";         BONDI_BLUE             = "\033[38;5;37m";         CERULEAN_BLUE       = "\033[38;5;38m"
    BLUE_BOLT             = "\033[38;5;39m";         STRONG_LIME_GREEN      = "\033[38;5;40m";         MALACHITE_GREEN     = "\033[38;5;41m"
    CARIBBEAN_GREEN       = "\033[38;5;42m";         STRONG_CYAN            = "\033[38;5;43m";         DARK_TURQUOISE      = "\033[38;5;44m"
    VIVID_SKY_BLUE        = "\033[38;5;45m";         ELECTRIC_GREEN         = "\033[38;5;46m";         SPRING_GREEN        = "\033[38;5;47m"
    GUPPIE_GREEN          = "\033[38;5;48m";         MEDIUM_SPRING_GREEN    = "\033[38;5;49m";         BRIGHT_TURQUOISE    = "\033[38;5;50m"
    AQUA                  = "\033[38;5;51m";         BLOOD_RED              = "\033[38;5;52m";         VERY_DARK_MAGENTA   = "\033[38;5;53m"
    INDIGO                = "\033[38;5;54m";         DARK_VIOLET            = "\033[38;5;55m";         LIGHT_VIOLET        = "\033[38;5;56m"
    ELECTRIC_INDIGO       = "\033[38;5;57m";         VERDUN_GREEN           = "\033[38;5;58m";         SCORPION_GRAY       = "\033[38;5;59m"
    UCLA_BLUE             = "\033[38;5;60m";         SCAMPI_BLUE            = "\033[38;5;61m";         SLATE_BLUE          = "\033[38;5;62m"
    CORNFLOWER_BLUE       = "\033[38;5;63m";         AVOCADO_GREEN          = "\033[38;5;64m";         GLADE_GREEN         = "\033[38;5;65m"
    STEEL_TEAL_CYAN       = "\033[38;5;66m";         STEEL_BLUE             = "\033[38;5;67m";         HAVELOCK_BLUE       = "\033[38;5;68m"
    BLUEBERRY             = "\033[38;5;69m";         KELLY_GREEN            = "\033[38;5;70m";         FOREST_GREEN        = "\033[38;5;71m"
    POLISHED_PIN_GREEN    = "\033[38;5;72m";         CRYSTAL_BLUE           = "\033[38;5;73m";         AQUA_PEARL          = "\033[38;5;74m"
    BLUE_JEANS            = "\033[38;5;75m";         HARLEQUIN_GREEN        = "\033[38;5;76m";         MODERATE_LIME_GREEN = "\033[38;5;77m"
    CARIBBEAN_GREEN_PEARL = "\033[38;5;78m";         EUCALYPTUS_GREEN       = "\033[38;5;79m";         MEDDIUM_TURQUOISE   = "\033[38;5;80m"
    MAYA_BLUE             = "\033[38;5;81m";         BRIGHT_GREEN           = "\033[38;5;82m";         LIGHT_LIME_GREEN    = "\033[38;5;83m"
    LIGHT_MALACHITE_GREEN = "\033[38;5;84m";         MEDDIUM_AQUAMARINE     = "\033[38;5;85m";         AQUAMARINE_GREEN    = "\033[38;5;86m"
    AQUAMARINE_CYAN       = "\033[38;5;87m";         DEEP_RED               = "\033[38;5;88m";         FRENCH_PLUM_VIOLET  = "\033[38;5;89m"
    FRESH_EGGPLANT_VIOLET = "\033[38;5;90m";         VIOLET                 = "\033[38;5;91m";         STRONG_VIOLET       = "\033[38;5;92m"
    ELECTRIC_VIOLET       = "\033[38;5;93m";         BROWN                  = "\033[38;5;94m";         COPPER_BROWN        = "\033[38;5;95m"
    MOSTLY_VIOLET         = "\033[38;5;96m";         ROYAL_PURPLE           = "\033[38;5;97m";         MEDDIUM_PURPLE      = "\033[38;5;98m"
    BLUEBERRY_PURPLE      = "\033[38;5;99m";         DARK_OLIVE_GREEN       = "\033[38;5;100m";        CLAY_CREEK_GREEN    = "\033[38;5;101m"
    TAUPE_GRAY            = "\033[38;5;102m";        COOL_GRAY              = "\033[38;5;103m";        CHETWODE_BLUE       = "\033[38;5;104m"
    VIOLET_BLUE           = "\033[38;5;105m";        APPLE_GREEN            = "\033[38;5;106m";        ASPARAGUS_GREEN     = "\033[38;5;107m"
    LEAF_GREEN            = "\033[38;5;108m";        GRAYISH_CYAN           = "\033[38;5;109m";        COBALT_BLUE         = "\033[38;5;110m"
    SKY_BLUE              = "\033[38;5;111m";        PISTACHIO_GREEN        = "\033[38;5;112m";        MANTIS_GREEN        = "\033[38;5;113m"
    PASTEL_GREEN          = "\033[38;5;114m";        PEARL_AQUA             = "\033[38;5;115m";        SLIGHTLY_CYAN       = "\033[38;5;116m"
    PALE_CYAN             = "\033[38;5;117m";        GREEN                  = "\033[38;5;118m";        LIGHT_GREEN         = "\033[38;5;119m"
    VERY_LIGHT_LIME_GREEN = "\033[38;5;120m";        MINT_GREEN             = "\033[38;5;121m";        AQUA_LIME_CYAN      = "\033[38;5;122m"
    LIGHT_CYAN            = "\033[38;5;123m";        DARK_RED               = "\033[38;5;124m";        DARK_PINK           = "\033[38;5;125m"
    DARK_MAGENTA          = "\033[38;5;126m";        HELIOTROPE_MAGENTA     = "\033[38;5;127m";        VIVID_PURPLE        = "\033[38;5;128m"
    ELECTRIC_PURPLE       = "\033[38;5;129m";        DARK_ORANGE_BROWN      = "\033[38;5;130m";        ELECTRIC_BROWN      = "\033[38;5;131m"
    DARK_MODERATE_PINK    = "\033[38;5;132m";        DARK_MODERATE_MAGENTA  = "\033[38;5;133m";        RICH_LILAC_VIOLET   = "\033[38;5;134m"
    LAVENDER_INDIGO       = "\033[38;5;135m";        PIRATE_GOLD_BROWN      = "\033[38;5;136m";        BRONZE_BROWN        = "\033[38;5;137m"
    DARK_GRAYISH_RED      = "\033[38;5;138m";        DARK_GRAYISH_MAGENTA   = "\033[38;5;139m";        LAVENDER            = "\033[38;5;140m"
    BRIGHT_LAVENDER       = "\033[38;5;141m";        LIGHT_GOLD_BROWN       = "\033[38;5;142m";        LIGHT_OLIVE_GREEN   = "\033[38;5;143m"
    DARK_GRAYISH_YELLOW   = "\033[38;5;144m";        SILVER_FOIL            = "\033[38;5;145m";        GRAYISH_BLUE        = "\033[38;5;146m"
    BLUE_PURPLE           = "\033[38;5;147m";        VIVID_LIME_GREEN       = "\033[38;5;148m";        MODERATE_GREEN      = "\033[38;5;149m"
    YELLOW_GREEN          = "\033[38;5;150m";        GRAYISH_LIME_GREEN     = "\033[38;5;151m";        CRYSTAL_CYAN        = "\033[38;5;152m"
    PALE_BLUE             = "\033[38;5;153m";        LIME                   = "\033[38;5;154m";        GREEN_YELLOW        = "\033[38;5;155m"
    VERY_LIGHT_GREEN      = "\033[38;5;156m";        MENTHOL_GREEN          = "\033[38;5;157m";        AEREO_BLUE          = "\033[38;5;158m"
    CELESTE_CYAN          = "\033[38;5;159m";        STRONG_RED             = "\033[38;5;160m";        ROYAL_RED           = "\033[38;5;161m"
    MEXICAN_PINK          = "\033[38;5;162m";        HOLLYWOOD_PINK         = "\033[38;5;163m";        STRONG_MAGENTA      = "\033[38;5;164m"
    PHLOX_VIOLET          = "\033[38;5;165m";        STRONG_ORANGE          = "\033[38;5;166m";        INDIAN_RED          = "\033[38;5;167m"
    BLUSH_RED             = "\033[38;5;168m";        SUPER_PINK             = "\033[38;5;169m";        ORCHID_MAGENTA      = "\033[38;5;170m"
    LIGHT_MAGENTA         = "\033[38;5;171m";        CHOCOLATE_BROWN        = "\033[38;5;172m";        COPPERFIELD_BROWN   = "\033[38;5;173m"
    SLIGHTLY_RED          = "\033[38;5;174m";        SLIGHTLY_PINK          = "\033[38;5;175m";        LIGHT_ORCHID_PINK   = "\033[38;5;176m"
    BRIGHT_LILAC_VIOLET   = "\033[38;5;177m";        MUSTARD_YELLOW         = "\033[38;5;178m";        EARTH_YELLOW        = "\033[38;5;179m"
    TAN_BROWN             = "\033[38;5;180m";        GRAYISH_RED            = "\033[38;5;181m";        GRAYISH_MAGENTA     = "\033[38;5;182m"
    PALE_VIOLET           = "\033[38;5;183m";        STRONG_YELLOW          = "\033[38;5;184m";        MODERATE_YELLOW     = "\033[38;5;185m"
    DECO_YELLOW           = "\033[38;5;186m";        PASTEL_GRAY            = "\033[38;5;187m";        LIGHT_SILVER        = "\033[38;5;188m"
    PALE_LAVENDER         = "\033[38;5;189m";        NEON_YELLOW            = "\033[38;5;190m";        LIGHT_GREEN_YELLOW  = "\033[38;5;191m"
    MINDARO_YELLOW        = "\033[38;5;192m";        PALE_GREEN             = "\033[38;5;193m";        VERY_PALE_GREEN     = "\033[38;5;194m"
    VERY_LIGHT_CYAN       = "\033[38;5;195m";        LIGHT_RED              = "\033[38;5;196m";        RASPBERRY_RED       = "\033[38;5;197m"
    BRIGHT_PINK           = "\033[38;5;198m";        PINK                   = "\033[38;5;199m";        MAGENTA             = "\033[38;5;200m"
    FUCHSIA               = "\033[38;5;201m";        BLAZE_ORANGE           = "\033[38;5;202m";        BITTERSWEET_RED     = "\033[38;5;203m"
    STRAWBERRY_RED        = "\033[38;5;204m";        HOT_PINK               = "\033[38;5;205m";        LIGHT_PINK          = "\033[38;5;206m"
    PINK_FLAMINGO         = "\033[38;5;207m";        DARK_ORANGE            = "\033[38;5;208m";        SALMON_ORANGE       = "\033[38;5;209m"
    TANGERINE_RED         = "\033[38;5;210m";        PINK_SALMON            = "\033[38;5;211m";        LAVENDER_ROSE       = "\033[38;5;212m"
    FUCHSIA_PINK          = "\033[38;5;213m";        ORANGE                 = "\033[38;5;214m";        LIGHT_ORANGE        = "\033[38;5;215m"
    VERY_LIGHT_ORANGE     = "\033[38;5;216m";        PALE_RED               = "\033[38;5;217m";        PALE_PINK           = "\033[38;5;218m"
    PALE_MAGENTA          = "\033[38;5;219m";        GOLD                   = "\033[38;5;220m";        DANDELION_YELLOW    = "\033[38;5;221m"
    JASMINE_BROWN         = "\033[38;5;222m";        PALE_ORANGE            = "\033[38;5;223m";        MISTY_ROSE_PINK     = "\033[38;5;224m"
    PINK_LACE             = "\033[38;5;225m";        YELLOW                 = "\033[38;5;226m";        LEMON_YELLOW        = "\033[38;5;227m"
    PASTEL_YELLOW         = "\033[38;5;228m";        PALE_YELLOW            = "\033[38;5;229m";        VERY_PALE_YELLOW    = "\033[38;5;230m"
    LIGHT_WHITE           = "\033[38;5;231m";        VAMPIRE_BLACK          = "\033[38;5;232m";        GRAY_BLACK          = "\033[38;5;233m"
    EERIE_BLACK           = "\033[38;5;234m";        RAISIN_BLACK           = "\033[38;5;235m";        DARK_CHARCOAL       = "\033[38;5;236m"
    BLACK_OLIVE           = "\033[38;5;237m";        OUTER_SPACE_GRAY       = "\033[38;5;238m";        DARK_LIVER_GRAY     = "\033[38;5;239m"
    DAVYS_GRAY            = "\033[38;5;240m";        GRANITE_GRAY           = "\033[38;5;241m";        DIM_GRAY            = "\033[38;5;242m"
    SONIC_SILVER          = "\033[38;5;243m";        GRAY                   = "\033[38;5;244m";        PHILIPPINE_GRAY     = "\033[38;5;245m"
    DUSTY_GRAY            = "\033[38;5;246m";        SPANISH_GRAY           = "\033[38;5;247m";        LIGHTISH_GRAY       = "\033[38;5;248m"
    PHILIPPINE_SILVER     = "\033[38;5;249m";        SILVER                 = "\033[38;5;250m";        SILVER_SAND         = "\033[38;5;251m"
    AMERICAN_SILVER       = "\033[38;5;252m";        ALTO_GRAY              = "\033[38;5;253m";        MERCURY_GRAY        = "\033[38;5;254m"
    DARK_WHITE            = "\033[38;5;255m";        DEFAULT                = "\033[39m";              OFF                 = "\033[39m"



class Style(enum.StrEnum):

    '''  This class uses the style name rather than the bool value.
         import custom_print as cp
         print(f"{cp.Fg.YELLOW} Hello {cp.Fg.OFF} Normal")
    '''

    BOLD_ON      = "\033[1m";       BOLD_OFF      = "\033[22m"
    DIM_ON       = "\033[2m";       DIM_OFF       = "\033[22m"
    ITALIC_ON    = "\033[3m";       ITALIC_OFF    = "\033[23m"
    UNDERLINE_ON = "\033[4m";       UNDERLINE_OFF = "\033[24m"
    BLINKING_ON  = "\033[5m";       BLINKING_OFF  = "\033[25m"
    INVERSE_ON   = "\033[7m";       INVERSE_OFF   = "\033[27m"
    HIDDEN_ON    = "\033[8m";       HIDDEN_OFF    = "\033[28m"
    STRIKE_ON    = "\033[9m";       STRIKE_OFF    = "\033[29m"
    RESET_ALL    = "\033[0m"
    OFF = "\033[22m"+"\033[23m"+"\033[24m"+"\033[25m"+"\033[27m"+"\033[28m"+"\033[29m"



def main():
    
    '''  Description of custom_print project  '''
    import custom_print as cp

    lst = [["Module Name",         "custom_print"                                   ],
           ["Version",             "1.1.4"                                          ],
           ["Author",              "Miguel Angel Aguilar Cuesta"                    ],
           ["Author Email",        "acma.mex@gmail.com"                             ],
           ["Description",         "Customized Print"                               ],
           ["Requirement",         "Python 3.12 or greater"                         ],
           ["Long Description",    "README.md"                                      ],
           ["Content Type",        "MarkDown"                                       ],
           ["Find README.md at",   "https://github.com/acma82/Custom_Print"         ],
           ["Dependencies",        "None"                                           ],
           ["License",             "Everyone Can Use It At Their Own Risk"          ]]

    tbl = cp.FancyFormat()
    FACE = " (" + "0" + chr(0x25E1) + "0" + ") "
    tbl.title_msg = FACE + "  Project Description "
    tbl.title_align = "center"
    tbl.title_bg = 231
    tbl.title_fg = 234
    tbl.title_bold = True


    tbl.footnote_msg = "Released on Friday, December 27, 2024"
    tbl.adj_top_space = 1
    tbl.adj_bottom_space = 1


    tbl.header_bg = 54;             tbl.data_bg = 231
    tbl.header_fg = 231;            tbl.data_fg = 234
    tbl.header_bold = True;         tbl.bold_data = True
    tbl.adj_top_margin = 2;         tbl.adj_indent = 4

    tbl.print_fancy_format(lst, "design_10")
    print()

if __name__ == "__main__":
    main()


'''
custom_print module can handle any type of variable.
'''

#pylint: disable=bare-except
#pylint: disable=invalid-name
#pylint: disable=unused-import
#pylint: disable=line-too-long
#pylint: disable=too-many-lines
#pylint: disable=no-else-return
#pylint: disable=unused-variable
#pylint: disable=too-many-locals
#pylint: disable=protected-access
#pylint: disable=too-many-branches
#pylint: disable=consider-using-in
#pylint: disable=chained-comparison
#pylint: disable=too-many-arguments
#pylint: disable=too-many-statements
#pylint: disable=multiple-statements
#pylint: disable=consider-using-join
#pylint: disable=unspecified-encoding
#pylint: disable=unnecessary-negation
#pylint: disable=singleton-comparison
#pylint: disable=too-few-public-methods
#pylint: disable=too-many-nested-blocks
#pylint: disable=too-many-public-methods
#pylint: disable=expression-not-assigned
#pylint: disable=consider-using-enumerate
#pylint: disable=unnecessary-comprehension
#pylint: disable=too-many-return-statements
#pylint: disable=unbalanced-tuple-unpacking
#pylint: disable=consider-using-max-builtin
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-positional-arguments
#pylint: disable=inconsistent-return-statements
#pylint: disable=possibly-used-before-assignment


#-----------------------------------------------------------------------------------------------------------------------------------------------------
#12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
#        1         2         3         4         5         6         7         8         9         A         B         C         D         E         F
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Required Modules                                                                                                                                   -
#-----------------------------------------------------------------------------------------------------------------------------------------------------
from custom_print.ref_names import Move

#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Cursor Class. Manipulate Cursor Around The Terminal                                                                                               --
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
class Cursor:
    '''
    Cursor class helps to move the cursor around the terminal
    '''
    def jumpTo(self,qty=0, direction=Move.DOWN):

        '''  Moves the cursor n position to the Direction Specified  '''

        print(Cursor.moveTo(self, qty, direction),end="")


    def moveTo(self,qty=0, direction=Move.DOWN):

        '''  Moves the cursor n position to the Direction Specified  '''

        if direction.lower() == Move.UP or direction.lower() == "u":
            if qty == 0: movement = ""
            else:        movement = f"\033[{str(qty)}A"

        elif direction.lower() == Move.DOWN  or direction.lower() == "d":
            if qty == 0: movement = ""
            else:        movement = f"\033[{str(qty)}B"

        elif direction.lower() == Move.RIGHT or direction.lower() == "r":
            if qty == 0: movement = ""
            else:        movement = f"\033[{str(qty)}C"

        elif direction.lower() == Move.LEFT or direction.lower() == "l":
            if qty == 0: movement = ""
            else:        movement = f"\033[{str(qty)}D"

        else:  movement = ""
        return movement


    def jumpxy(self,x=0,y=0):

        '''  This function moves the cursor to specific position (x,y)  '''

        print(Cursor.movexy(self, y, x),end="")


    def movexy(self,x=0, y=0):

        '''  Moves the cursor to specific position (x, y)  '''

        if (y<=-1 or x<=-1):
            posi = ""
        else:
            posi = f"\033[{str(y)};{str(x)}H"

        return posi



'''
custom_print module can handle any type of variable.
'''

#pylint: disable=bare-except
#pylint: disable=invalid-name
#pylint: disable=unused-import
#pylint: disable=line-too-long
#pylint: disable=too-many-lines
#pylint: disable=no-else-return
#pylint: disable=wildcard-import
#pylint: disable=unused-variable
#pylint: disable=too-many-locals
#pylint: disable=protected-access
#pylint: disable=too-many-branches
#pylint: disable=consider-using-in
#pylint: disable=chained-comparison
#pylint: disable=too-many-arguments
#pylint: disable=too-many-statements
#pylint: disable=multiple-statements
#pylint: disable=consider-using-join
#pylint: disable=unspecified-encoding
#pylint: disable=unnecessary-negation
#pylint: disable=singleton-comparison
#pylint: disable=too-few-public-methods
#pylint: disable=too-many-nested-blocks
#pylint: disable=too-many-public-methods
#pylint: disable=expression-not-assigned
#pylint: disable=consider-using-enumerate
#pylint: disable=unnecessary-comprehension
#pylint: disable=too-many-return-statements
#pylint: disable=unbalanced-tuple-unpacking
#pylint: disable=consider-using-max-builtin
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-instance-attributes
#pylint: disable=too-many-positional-arguments
#pylint: disable=inconsistent-return-statements
#pylint: disable=possibly-used-before-assignment


#-----------------------------------------------------------------------------------------------------------------------------------------------------
#12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
#        1         2         3         4         5         6         7         8         9         A         B         C         D         E         F
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Required Modules                                                                                                                                   -
#-----------------------------------------------------------------------------------------------------------------------------------------------------
import os
from custom_print.fancy_cursor import*            # cursor already import Move Class

from custom_print.ref_names import Align
from custom_print.ref_names import Length_bg
from custom_print.ref_names import Line_Style

from custom_print.fancy_format import FancyFormat

from custom_print.fancy_functions import ins_chr
from custom_print.fancy_functions import set_font
from custom_print.fancy_functions import reset_font
from custom_print.fancy_functions import move_cursor_right
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Fancy Message Class (Single line or a Paragraph Text in the Terminal)                                                                             --
#-----------------------------------------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------------------------------------
class FancyMessage(Cursor):
    '''
    FancyMessage class
    '''
    def __init__(self):
        super().__init__()          # Super Class to use all (vars and funs) from Cursor Class
                                    # with the Initialization Draw Class(self), ex. self.gotoxy(x,y)
        self.body_bg        = 4;          self.body_underline = False     # 4         False
        self.body_fg        = 231;        self.body_blinking  = False     # 231       False
        self.body_bold      = False;      self.body_inverse   = False     # False     False
        self.body_dim       = False;      self.body_hidden    = False     # False     False
        self.body_italic    = False;      self.body_strike    = False     # False     False

        self.body_msg = "Body Msg";       self.help_lines = False

        self.left_indent = 2;             self.right_indent = 2
        self.top_lines = 1;               self.bottom_lines = 1

        self.length = Length_bg.ALL_ROW
        # These two options don't do anything when length = Length_bg.All_ROW
        self.adj_bg_lines_to_right_indent = False     # True or False
        self.adj_bg_msg_to_space_available = False    # True or False



        #--------------------------------------------------------------------
        # Note Settings Here, print_fancy_note
        self.note_msg = " Note: "
        self.note_align = Align.JUSTIFY;    self.note_position = 1
        self.note_bg = 231;                 self.note_fg = 0;                 self.note_bold      = False
        self.note_dim = False;              self.note_italic  = False;        self.note_underline = False
        self.note_blinking = False;         self.note_inverse = False;        self.note_hidden    = False
        self.note_strike = False;           self.note_left_space = 2;         self.note_right_space = 2

        # Title Settings Here, print_fancy_message
        self.title_align = Align.LEFT;      self.title_indent = 2;            self.title_msg = "" # title_indent works with Align.JUSTIFY
        self.lines_title_body = 1;          self.title_strike = False
        self.title_bg = 4;                  self.title_fg = 231;              self.title_bold  = False
        self.title_dim = False;             self.title_italic = False;        self.title_underline = False
        self.title_blinking = False;        self.title_inverse = False;       self.title_hidden = False

        # Footnote Settings Here, print_fancy_message
        self.footnote_align = Align.RIGHT;  self.footnote_indent = 2;         self.footnote_msg = "" # footnote_indent works with Align.JUSTIFY
        self.footnote_lines_body = 1;       self.footnote_strike = False
        self.footnote_bg = 4;               self.footnote_fg = 231;           self.footnote_bold  = False
        self.footnote_dim = False;          self.footnote_italic = False;     self.footnote_underline = False
        self.footnote_blinking = False;     self.footnote_inverse = False;    self.footnote_hidden = False


    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Get Message Attributes                                                                                                                         -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    def get_msg_attributes(self,data:str="Message",all_attribute:bool=False):
        '''
            getting the attributes from the message
        '''
        msg = str(data)
        tncols, tnrows = os.get_terminal_size()

        space_available = tncols - self.left_indent - self.right_indent

        longest_line   = 0;                       quotient = 0
        letter_counter = 0;                       residue  = 0
        msg_type       = "single_line";           new_msg  = ""


        quotient_number_letter_line_list = [];    fit_number_letter_line_list   = []
        residue_number_letter_line_list  = [];    carry_number_letter_line_list = []

        adj_diff_space     = [];                  number_letter_line_list = []
        result_multi_lines = []


        for l in msg:
            if l=="\n":
                number_letter_line_list.append(letter_counter)
                letter_counter = 0
                msg_type="multiple_lines"

            else:
                new_msg += l
                letter_counter += 1


        if msg_type == "single_line":
            quotient = int(letter_counter/space_available)
            residue  = letter_counter%space_available
            while quotient>0:
                number_letter_line_list.append(space_available)
                quotient -= 1
            number_letter_line_list.append(residue)

            for n in number_letter_line_list:
                adj_diff_space.append(space_available - n)


        else:   # multiple lines
            number_letter_line_list.append(letter_counter) # the last one not added
            longest_line = max(number_letter_line_list)
            # first item when only enter it's deleted
            if number_letter_line_list[0] == 0:  number_letter_line_list.pop(0)
            # last item when only enter it's deleted
            if number_letter_line_list[(len(number_letter_line_list))-1] == 0:
                number_letter_line_list.pop((len(number_letter_line_list))-1)

            if space_available > longest_line:
                for n in number_letter_line_list:
                    adj_diff_space.append(space_available-n)

            else:
                for line in range(len(number_letter_line_list)):
                    if number_letter_line_list[line] <= space_available:
                        fit_number_letter_line_list.append(number_letter_line_list[line])

                    else:
                        quotient = int(number_letter_line_list[line]/space_available)
                        residue  = number_letter_line_list[line]%space_available
                        n = quotient

                        while n > 0:
                            quotient_number_letter_line_list.append(space_available)
                            n -= 1

                        residue_number_letter_line_list.append(residue)
                        carry_number_letter_line_list.append(quotient+1)

                ctrl = 0
                for r in number_letter_line_list:
                    if r > space_available:
                        last_one = carry_number_letter_line_list[ctrl] - 1

                        for n in range(carry_number_letter_line_list[ctrl]):
                            if last_one == n:
                                result_multi_lines.append(residue_number_letter_line_list[ctrl])
                                ctrl += 1
                            else:
                                result_multi_lines.append(space_available)
                    else:
                        result_multi_lines.append(r)


                number_letter_line_list = result_multi_lines

                for n in number_letter_line_list:
                    adj_diff_space.append(space_available - n)

        if all_attribute == True:
            return tnrows, tncols, space_available, number_letter_line_list, adj_diff_space, new_msg, len(number_letter_line_list)

        else:
            return len(number_letter_line_list), space_available, tncols


    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Send the Data To the Terminal                                                                                                                  -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    def send_msg_terminal(self,data="Message"):
        '''
            printing the message
        '''
        def print_bg_lines(lines, bg_format_line_color="\033[0m"):
            if lines == 0:
                print("\033[0m",end="")
            else:
                n = lines
                while n>0:
                    print(bg_format_line_color)
                    n -= 1

        tnrows, tncols, space_available, number_letter_line_list, adj_diff_space, new_msg, n_lines = FancyMessage.get_msg_attributes(self,data,True)

        color = set_font(self.body_bold, self.body_bg, self.body_fg, self.body_italic, self.body_underline, self.body_strike,
                         self.body_blinking, self.body_dim, self.body_hidden, self.body_inverse)
        color2= set_font(bg=self.body_bg, fg=self.body_fg, inverse=self.body_inverse)

        # from here we need: tncols, space_available, number_letter_line_list, adj_diff_space, new_msg
        longest_line = max(number_letter_line_list)

        # self.adj_bg_lines_to_right_indent by default  = False
        # self.adj_bg_msg_to_space_available by default = False

        if self.length == Length_bg.ALL_ROW:
            bg_format_line_color = f"{color2}{ins_chr(tncols)}{reset_font()}"
            # change color for color2 to delete at the beginning the strike, and/or underline option(s)
            start_line = f"{color2}{ins_chr(self.left_indent)}"

        elif self.length == Length_bg.ONLY_WORD:
            if self.adj_bg_lines_to_right_indent == True:
                bg_format_line_color = f"{color2}{move_cursor_right(self.left_indent)}{ins_chr(space_available)}{reset_font()}"  # change color for color2

            else:  # elif (self.adj_bg_lines_to_right_indent == False):
                bg_format_line_color = f"{move_cursor_right(self.left_indent)}{color2}{ins_chr(longest_line)}{reset_font()}"     # change color for color2

            start_line = f"{move_cursor_right(self.left_indent)}{color2}"                                                        # change color for color2

        else: pass

        carry = 0; last_one = n_lines - 1
        print_bg_lines(self.top_lines, bg_format_line_color)       # bg_line

        print(start_line,end="")

        # start printing the message
        for nl in range(n_lines):
            for n in range(number_letter_line_list[nl]):
                print(f"{color}{new_msg[carry+n]}",end="")          # added color because the color2 can be slightly different

            carry += number_letter_line_list[nl]

            if self.length == Length_bg.ALL_ROW:
                for n in range(adj_diff_space[nl]+self.right_indent):
                    print(color2+" ",end="")                        # to delete at the end the strike, and/or underline option(s)

            elif self.length == Length_bg.ONLY_WORD:
                if self.adj_bg_msg_to_space_available == True:
                    for n in range(space_available -  number_letter_line_list[nl]):
                        print(color2+" ",end="")                    # to delete the strike we add color2
                else:                                               # elif (self.adj_bg_msg_to_space_available == False):
                    for n in range(longest_line-number_letter_line_list[nl]):
                        print(color2+" ",end="")

                print(f"{reset_font()}",end="")

            else:  pass

            print()
            if last_one == nl: pass
            else:                print(start_line,end="")

        # end printing the message
        print_bg_lines(self.bottom_lines, bg_format_line_color)    # bg_line


    #---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Print Fancy Note                                                                                                                                 -
    #---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    def print_fancy_note(self, body_msg:str="")->None:
        '''
            It prints the fancy note with the attributes defined
        '''

        if body_msg == "":  body_msg = self.body_msg

        # save original values
        li_obj = self.left_indent

        # settings for the body_msg
        if self.note_msg == "":  len_note_msg = 0
        else:                      len_note_msg = len(self.note_msg)

        self.left_indent = self.note_left_space + len_note_msg + self.note_right_space
        n_lines, space_available, tncols = self.get_msg_attributes(body_msg, False)

        total_back_lines = self.top_lines + n_lines + self.bottom_lines
        if   self.note_position >= (total_back_lines): lines_back = 0
        elif self.note_position <= 0:                  lines_back = total_back_lines
        else:                                          lines_back = total_back_lines - self.note_position

        self.send_msg_terminal(body_msg)

        # settings for the note
        settings_note = set_font(bold=self.note_bold, bg=self.note_bg, fg=self.note_fg, italic=self.note_italic,\
                                 underline=self.note_underline, strike=self.note_strike, blinking=self.note_blinking,\
                                 dim=self.note_dim, hidden=self.note_hidden, inverse=self.note_inverse)

        if self.note_align == Align.LEFT or self.note_align == "l":
            print(f"{self.moveTo(qty=lines_back, direction=Move.UP)}{settings_note}{self.note_msg}",end="")

        elif self.note_align == Align.CENTER or self.note_align == "c":
            myq = int((self.note_left_space+self.note_right_space)/2)
            print(f"{self.moveTo(qty=lines_back, direction=Move.UP)}{self.moveTo(myq, Move.RIGHT)}{settings_note}{self.note_msg}",end="")

        elif self.note_align == Align.RIGHT or self.note_align == "r":
            myq = self.note_left_space + self.note_right_space
            print(f"{self.moveTo(lines_back, Move.UP)}{self.moveTo(myq, Move.RIGHT)}{settings_note}{self.note_msg}",end="")

        else:  # JUSTIFY
            print(f"{self.moveTo(lines_back, Move.UP)}{self.moveTo(self.note_left_space, Move.RIGHT)}{settings_note}{self.note_msg}")

        self.jumpTo(qty=lines_back-1, direction=Move.DOWN)
        print(f"{reset_font()}",end="")

        # putting back original values
        self.left_indent = li_obj
        # n_lines, space_available, tncols are variables for reference to calculate the message
        if self.help_lines == True:
            print(f"{ins_chr(self.left_indent)}Body_Lines:{n_lines}  Space_Available:{space_available}  N.Cols: {tncols}N.Lines:{total_back_lines}")


    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Print Fancy Message                                                                                                                            -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    def print_fancy_message(self, body_msg:str="")->None:

        '''  It prints the fancy message with the attributes defined  '''

        if body_msg == "":  body_msg = self.body_msg

        # save original values
        li_obj = self.left_indent;      bold_obj    = self.body_bold;            blinking_obj  = self.body_blinking
        tl_obj = self.top_lines;        italic_obj  = self.body_italic;          underline_ojb = self.body_underline
        bg_obj = self.body_bg;          strike_obj  = self.body_strike;          fnm_obj       = self.footnote_msg
        fg_obj = self.body_fg;          hidden_obj  = self.body_hidden;          dim_obj       = self.body_dim
        bl_obj = self.bottom_lines;     inverse_obj = self.body_inverse
        ti_obj = self.title_msg

        n_lines, space_available, tncols = self.get_msg_attributes(body_msg)  # settings for title

        #---------------------------------------------------------------------------------------------------------------------------------------------
        #---------------------------------------------------------------------------------------------------------------------------------------------
        if not self.title_msg == "":
            # working with the font color
            self.body_bg     = self.title_bg;          self.body_underline = self.title_underline
            self.body_fg     = self.title_fg;          self.body_blinking  = self.title_blinking
            self.body_bold   = self.title_bold;        self.body_inverse   = self.title_inverse
            self.body_dim    = self.title_dim;         self.body_hidden    = self.title_hidden
            self.body_italic = self.title_italic;      self.body_strike    = self.title_strike

            if   self.title_align == Align.LEFT or self.title_align == "l":  pass

            elif self.title_align == Align.CENTER or self.title_align == "c":
                sp = int((space_available - len(self.title_msg))/2)
                self.title_msg = ins_chr(sp) + self.title_msg

            elif self.title_align == Align.RIGHT or self.title_align == "r":
                sp = space_available - len(self.title_msg) # 1 for not jumping line and finished
                self.title_msg = ins_chr(sp) + self.title_msg

            else:                                          # Align.JUSTIFY
                self.title_msg = ins_chr(self.title_indent) + self.title_msg

            self.bottom_lines = self.lines_title_body
            self.send_msg_terminal(self.title_msg)

            # This is necessary because when is right alignment, it jumps automatically to the next row
            if (self.title_align == Align.RIGHT and self.title_msg != ""):
                print("\033[1A",end="")
                print(f"{ins_chr(tncols)}")
                print("\033[1A",end="")

            # settings for body (we recovered left_indent, and change bottom_lines and change top_lines)
            if not self.footnote_msg == "":  self.bottom_lines = 0
            else:                            self.bottom_lines = bl_obj

            self.left_indent = li_obj
            self.body_bg     = bg_obj;          self.body_underline = underline_ojb
            self.body_fg     = fg_obj;          self.body_blinking  = blinking_obj
            self.body_bold   = bold_obj;        self.body_inverse   = inverse_obj
            self.body_dim    = dim_obj;         self.body_hidden    = hidden_obj
            self.body_italic = italic_obj;      self.body_strike    = strike_obj

            if not self.title_msg == "":  self.top_lines = 0
            else:                         self.top_lines = tl_obj

            self.body_fg = fg_obj  # returning the color for the body
            self.send_msg_terminal(body_msg)

        else:
            if not self.footnote_msg == "":   self.bottom_lines = self.footnote_lines_body
            else:                             self.bottom_lines = bl_obj

            self.send_msg_terminal(body_msg)

        #---------------------------------------------------------------------------------------------------------------------------------------------
        #---------------------------------------------------------------------------------------------------------------------------------------------
        if not self.footnote_msg == "":
            if   self.footnote_align == Align.LEFT or self.footnote_align == "l":  pass

            elif self.footnote_align == Align.CENTER or self.footnote_align == "c":
                sp = int((space_available - len(self.footnote_msg))/2)
                self.footnote_msg = ins_chr(sp) + self.footnote_msg

            elif self.footnote_align == Align.RIGHT or self.footnote_align == "r":
                sp = space_available - len(self.footnote_msg) # 1 for not jumping line and finished
                self.footnote_msg = ins_chr(sp) + self.footnote_msg

            else:
                self.footnote_msg = ins_chr(self.footnote_indent) + self.footnote_msg # JUSTIFY

            self.top_lines    = self.footnote_lines_body;    self.bottom_lines   = bl_obj
            self.body_bg      = self.footnote_bg;            self.body_underline = self.footnote_underline
            self.body_fg      = self.footnote_fg;            self.body_blinking  = self.footnote_blinking
            self.body_bold    = self.footnote_bold;          self.body_inverse   = self.footnote_inverse
            self.body_dim     = self.footnote_dim;           self.body_hidden    = self.footnote_hidden
            self.body_italic  = self.footnote_italic;        self.body_strike    = self.footnote_strike

            self.send_msg_terminal(self.footnote_msg)

            # This is necessary because when is right alignment, it jumps automatically to the next row
            if self.footnote_align == Align.RIGHT and self.footnote_msg != "":
                print("\033[1A",end="")
                print(f"{ins_chr(tncols)}")
                print("\033[1A",end="")

        else:  pass

        # putting back original values
        self.top_lines    = tl_obj;            self.left_indent    = li_obj            #  self.bottom_lines = bl_obj
        self.body_bg      = bg_obj;            self.body_underline = underline_ojb
        self.body_fg      = fg_obj;            self.body_blinking  = blinking_obj
        self.body_bold    = bold_obj;          self.body_inverse   = inverse_obj
        self.body_dim     = dim_obj;           self.body_hidden    = hidden_obj
        self.body_italic  = italic_obj;        self.body_strike    = strike_obj
        self.footnote_msg = fnm_obj;           self.title_msg      = ti_obj

        # n_lines, space_available, tncols are variables for reference to calculate the message
        if self.help_lines == True:
            total_lines = n_lines + self.top_lines + self.bottom_lines

            if self.title_msg != "":     total_lines += 1 + self.lines_title_body

            if self.footnote_msg != "":  total_lines += 1 + self.footnote_lines_body

            print(f"{ins_chr(self.left_indent)}Body_Lines:{n_lines}  Space_Available:{space_available}  N.Cols: {tncols}  N.Lines:{total_lines}")


    #-------------------------------------------------------------------------------------------------------------------------------------------------
    # Get Message Attributes                                                                                                                         -
    #-------------------------------------------------------------------------------------------------------------------------------------------------
    def get_message_attributes(self, body_msg:str="", print_attributes=True)->list:
        '''
        It returns the attributes of the message
        '''
        tnrows, tncols, space_available, number_letter_line_list, adj_diff_space, new_msg_list, n_lines =\
                                                                                                FancyMessage.get_msg_attributes(self,body_msg,True)

        if body_msg == "":  body_msg = self.body_msg

        smallest_line = min(number_letter_line_list)
        longest_line  = max(number_letter_line_list)
        words = body_msg.split()
        counter_words = len(words)
        total_characters = sum(number_letter_line_list)
        screen_size_xy = [tncols,tnrows]

        result_lst  =  [["Attributes",                    "Values"],
                        ["Screen Size_xy",                screen_size_xy],
                        ["Left Indent",                   self.left_indent],
                        ["Right Indent",                  self.right_indent],
                        ["Space Available",               space_available],
                        ["Longest Line",                  longest_line],
                        ["Smallest Line",                 smallest_line],
                        ["List Line Lengths",             number_letter_line_list],
                        ["List Line Spaces",              adj_diff_space],
                        ["Words Into a List",             "\'words\'"],
                        ["Total Number of Lines",         n_lines],
                        ["Total Number of Words",         counter_words],
                        ["Total Number of Characters",    total_characters]]


        new_msg_list = [["Position","Word"]]
        cnt = 0
        for n in words:
            new_msg_list.append([cnt, n])
            cnt += 1

        if print_attributes == True:
            tbl = FancyFormat()
            # Title
            tbl.title_msg = "  get_message_attributes(message, True)  "
            tbl.title_align = Align.LEFT
            tbl.title_bold   = True;   tbl.title_bg = 231
            tbl.title_italic = True;   tbl.title_fg = 4
            # bg colors
            tbl.horizontal_line_bg = 4
            tbl.vertical_line_bg   = 4
            tbl.outer_corner_bg         = 4

            tbl.inner_corner_bg      = 4
            tbl.header_horizontal_line_bg = 4

            tbl.header_corner_bg = 4
            tbl.header_vertical_line_bg     = 4

            tbl.header_bg = 90
            tbl.header_fg = 231
            tbl.header_bold = True

            tbl.data_bg = 231
            tbl.data_fg = 0
            tbl.data_bold = True

            tbl.adj_top_margin = 2
            tbl.adj_indent = 4
            tbl.adj_space  = 4

            tbl.header_horizontal_line_on = True
            # tbl.horizontal_line_on = False
            tbl.middle_horizontal_line_on = False
            tbl.adj_bottom_margin = 1
            tbl.print_fancy_format(data=result_lst, style=Line_Style.SINGLE_SPACE)
            tbl.adj_top_margin = 1
            tbl.title_msg = "  Words of The Message Into a List  "
            tbl.print_fancy_format(new_msg_list)

        return result_lst, new_msg_list
